#ifndef KRY_AUDIO_OPENAL_SOURCE_H
#define KRY_AUDIO_OPENAL_SOURCE_H

#include "Utilities\Vector.h"
#include <vector>

namespace kry
{
	namespace Audio
	{
		class Buffer;

		enum class Status : unsigned char
		{
			PLAYING = 0,
			PAUSED,
			STOPPED
		};

		class Source /** #TODO(incomplete) add queue queries (queued and processed) http://www.openal.org/documentation/openal-1.1-specification.pdf */
		{
			public:
				Source();
				Source(const Buffer& buffer);
				Source(const Source& copy);
				Source(Source&& move);
				~Source();
				void setPosition(const Util::Vector3f& newposition);
				void setVelocity(const Util::Vector3f& newvelocity);
				void setVolume(float newvolume);
				void setPitch(float newpitch);
				void setDistance(float newdistance);
				void setAttenuation(float newattenuation);
				void setNewBuffer(const Buffer& buffer);
				void setLooping(bool loop);
				void setOffset(float secondsoffset);
				float getOffset() const;
				void play();
				void pause();
				void stop();
				void queue(const Buffer& buffer);
				void unqueue(const Buffer& buffer);
				void clearQueue();
				Status getStatus() const;
				const Buffer* getCurrBuffer() const;
				unsigned getCurrBufferIndex() const;

				Source& operator=(const Source& copy);
				Source& operator=(Source&& move);
				
				inline const std::vector<Buffer*>& getQueue() const;
				inline const Util::Vector3f& getPosition() const;
				inline const Util::Vector3f& getVelocity() const;
				inline float getVolume() const;
				inline float getPitch() const;
				inline float getDistance() const;
				inline float getAttenuation() const;
				inline unsigned getID() const;
				inline bool isLooping() const;
				inline void bind();
				inline void unbind();

			private:
				void destroy();
				
				std::vector<Buffer*> bufferqueue;
				Util::Vector3f position;
				Util::Vector3f velocity;
				float volume;
				float pitch;
				float distance;
				float attenuation;
				unsigned id;
				bool looping;
				int binds;
		};
		

		const std::vector<Buffer*>& Source::getQueue() const
		{
			return bufferqueue;
		}

		const Util::Vector3f& Source::getPosition() const
		{
			return position;
		}

		const Util::Vector3f& Source::getVelocity() const
		{
			return velocity;
		}

		float Source::getVolume() const
		{
			return volume;
		}

		float Source::getPitch() const
		{
			return pitch;
		}

		float Source::getDistance() const
		{
			return distance;
		}

		float Source::getAttenuation() const
		{
			return attenuation;
		}

		unsigned Source::getID() const
		{
			return id;
		}

		bool Source::isLooping() const
		{
			return looping;
		}

		void Source::bind()
		{
			++binds;
		}

		void Source::unbind()
		{
			--binds;
		}
	}
}

#endif
