#ifndef KRY_GRAPHICS_CANVAS_H
#define KRY_GRAPHICS_CANVAS_H

#include "Utilities\Vector.h"
#include "Graphics\Primitives.h"
#include <vector>
#include <map>

namespace kry
{
	namespace Graphics
	{
		enum class GraphicObjectType : char
		{
			SPRITE,
			TEXT,
			QUAD,
			LINE,
			TRIANGLE,
			ELLIPSE
		};

		class Renderer;

		class Canvas
		{
		public:
			Canvas();
			~Canvas();

			void clear();
			void addSprite(Sprite* sprite);
			void addSprites(const std::vector<Sprite*>& sprites);
			void addText(Text* text);
			void addTexts(const std::vector<Text*>& texts);
			void addQuad(Quad* quad);
			void addQuads(const std::vector<Quad*>& quads);
			void addLine(Line* line);
			void addLines(const std::vector<Line*>& lines);
			void addTriangle(Triangle* triangle);
			void addTriangles(const std::vector<Triangle*>& triangles);
			void addEllipse(Ellipse* ellipse);
			void addEllipses(const std::vector<Ellipse*>& ellipses);

			void setResolution(const Util::Vector2i& Resolution);
			Util::Vector2i getResolution() const;
			Util::Vector2f getCoord(const Util::Vector2f& Coord) const;

			// Public transformation members
			Util::Vector2f position;
			Util::Vector2f pan;
			Util::Vector2f scale;
			Util::Vector2f dimensions;			
			Util::Vector4f rgba;
			float rotation;
			bool linearFilter;

		protected:
			void deleteBuffers();

			friend class Renderer;
			std::vector<GraphicObjectType> typeBatch;
			std::vector<void*> graphicBatch;
			std::map<Text*, std::vector<Sprite*> > glyphs;

			unsigned int fboID;
			unsigned int colorID;
			Util::Vector2i resolution;
			Util::Vector2f min;
			Util::Vector2f max;
			unsigned int batchCapacityCache;
		};
	}
}

#endif
