#ifndef KRY_NETWORK_NETWORKDEVICE_H
#define KRY_NETWORK_NETWORKDEVICE_H

#include "Utilities\String.h"
#include <memory>

namespace kry
{
	namespace Network
	{
		class NetworkDevice
		{
			public:
				~NetworkDevice();
				void startNetworking();
				void stopNetworking();

				inline bool hasStarted() const;
				inline unsigned short getMajorVersion() const;
				inline unsigned short getMinorVersion() const;
				inline unsigned short getMaxSockets() const;
				inline unsigned short getMaxUDPSize() const;
				inline const Util::String& getDescription() const;
				inline const Util::String& getSystemStatus() const;

				static NetworkDevice* getDevice();

			private:
				NetworkDevice();
				NetworkDevice(const NetworkDevice& device);
				NetworkDevice& operator=(const NetworkDevice& device);

				bool started;
				unsigned short majorversion;
				unsigned short minorversion;
				unsigned short maxsockets;
				unsigned short maxudpsize;
				Util::String description;
				Util::String systemstatus;

				static std::unique_ptr<NetworkDevice> single;
		};


		bool NetworkDevice::hasStarted() const
		{
			return started;
		}

		unsigned short NetworkDevice::getMajorVersion() const
		{
			return majorversion;
		}

		unsigned short NetworkDevice::getMinorVersion() const
		{
			return minorversion;
		}

		unsigned short NetworkDevice::getMaxSockets() const
		{
			return maxsockets;
		}

		unsigned short NetworkDevice::getMaxUDPSize() const
		{
			return maxudpsize;
		}

		const Util::String& NetworkDevice::getDescription() const
		{
			return description;
		}

		const Util::String& NetworkDevice::getSystemStatus() const
		{
			return systemstatus;
		}
	}
}

#endif
