#ifndef KRY_NETWORK_TCPSERVER_H
#define KRY_NETWORK_TCPSERVER_H

#include "Utilities\StringTypes.h"
#include <functional>
#include <memory>

namespace kry
{
	namespace Network
	{
		class TCPSocket;
		class TCPServer;

		typedef std::function<void(TCPServer& sender, std::shared_ptr<TCPSocket>& client)> ServerEvent;

		class TCPServer  /** #TODO(incomplete) add copy/move constructors/operators */
		{
			public:
				TCPServer(unsigned short port);
				~TCPServer();

				void bind(unsigned short port);
				void listen(unsigned backlog);
				void stopListening();
				bool accept(); // returns true upon a successful connection
				void close();
				void setReuseAddress(bool reuse);
				unsigned getBoundPort() const;
				bool isClosed() const;

				inline void onAccept(const ServerEvent& onaccept);
				inline unsigned getBacklogCount() const;
				inline bool isReusingAddress() const;
				inline bool isListening() const;

			private:
				int attemptBind();

				ServerEvent acceptevent;
				TCPSocket* socket;
				unsigned maxconnections;
				bool reuseaddress;
				bool listening;
		};


		void TCPServer::onAccept(const ServerEvent& onaccept)
		{
			acceptevent = onaccept;
		}

		unsigned TCPServer::getBacklogCount() const
		{
			return maxconnections;
		}

		bool TCPServer::isReusingAddress() const
		{
			return reuseaddress;
		}

		bool TCPServer::isListening() const
		{
			return listening;
		}
	}
}

#endif
