#ifndef KRY_NETWORK_TCPSOCKET_H
#define KRY_NETWORK_TCPSOCKET_H

#include "Network\SocketDescriptor.h"
#include "Utilities\String.h"

namespace kry
{
	namespace Network
	{
		class Packet;

		class TCPSocket /** #TODO(incomplete) add stream operator overloads */
		{
			public:
				TCPSocket(const Util::String& address, unsigned short port = 80);
				TCPSocket(SocketDescriptor descriptor, const SocketAddress& sockaddress);
				TCPSocket(const TCPSocket& tcpsocket);
				TCPSocket(TCPSocket&& tcpsocket);
				~TCPSocket();

				void open(const Util::String& address, unsigned short port = 80);
				void connect();
				void disconnect();
				void close();
				void send(const Packet& packet); /** #TODO(refactor) getsockopt for sol_recvbuf could make this redundant */
				void sendRaw(const char* data, unsigned length);
				void receive(Packet& packet);
				bool receiveRaw(char* data, unsigned length); // used for data that might be larger than the buffer size (>=4mb), true if there is more to go
				void setSendTimeout(unsigned timeout);
				void setRecvTimeout(unsigned timeout);

				inline bool isClosed() const;
				inline bool isConnected() const;
				inline const Util::String& getAddress() const;
				inline unsigned short getPort() const;
				inline unsigned getSendTimeout() const;
				inline unsigned getRecvTimeout() const;
				inline SocketDescriptor getDescriptor() const;
				inline const AddressInfo getAddressInfo() const;

				TCPSocket& operator=(const TCPSocket& copy);
				TCPSocket& operator=(TCPSocket&& move);

			private:
				void openFromDetails();
				void makeAddressInfo(const Util::String& addr, const Util::String& prt);
				void destroy();

				Util::String address;
				AddressInfo info;
				SocketDescriptor socket;
				unsigned sendtimeout;
				unsigned recvtimeout;
				unsigned short port;
				bool closed;
				bool connected;
		};


		bool TCPSocket::isClosed() const
		{
			return closed;
		}

		bool TCPSocket::isConnected() const
		{
			return connected;
		}

		const Util::String& TCPSocket::getAddress() const
		{
			return address;
		}

		unsigned short TCPSocket::getPort() const
		{
			return port;
		}

		unsigned TCPSocket::getSendTimeout() const
		{
			return sendtimeout;
		}

		unsigned TCPSocket::getRecvTimeout() const
		{
			return recvtimeout;
		}

		SocketDescriptor TCPSocket::getDescriptor() const
		{
			return socket;
		}

		const AddressInfo TCPSocket::getAddressInfo() const
		{
			return info;
		}
	}
}

#endif
