#ifndef KRY_NETWORK_UDPSOCKET_H
#define KRY_NETWORK_UDPSOCKET_H

#include "Network\SocketDescriptor.h"
#include "Utilities\String.h"

namespace kry
{
	namespace Network
	{
		class Packet;
		class DatagramPacket;

		class UDPSocket /** #TODO(incomplete) add stream operator overloads */
		{
			public:
				UDPSocket(unsigned short port, const Util::String& localaddress = ANY_ADDRESS, unsigned sendtimeout = 30000, unsigned recvtimeout = 30000);
				UDPSocket(const UDPSocket& copy);
				UDPSocket(UDPSocket&& move);
				~UDPSocket();

				void bind(unsigned short port, const Util::String& localaddress = ANY_ADDRESS);
				void send(const DatagramPacket& packet);
				void send(const Packet& packet, const Util::String& ipaddress, unsigned short port);
				bool receive(DatagramPacket& packet);
				void close();
				Util::String getBoundAddress() const;
				unsigned short getBoundPort() const;
				void setReuseAddress(bool reuse); // using these setters resets the socket, just a lil warning
				void setSendTimeout(unsigned timeout);
				void setRecvTimeout(unsigned timeout);

				UDPSocket& operator=(const UDPSocket& copy);
				UDPSocket& operator=(UDPSocket&& move);

				inline bool isClosed() const;
				inline bool isReusingAddress() const;
				inline unsigned getSendTimeout() const;
				inline unsigned getRecvTimeout() const;
				inline SocketDescriptor getDescriptor() const;

				static const Util::String ANY_ADDRESS;

			private:
				void openSocket();
				int attemptBind();

				SocketAddress address;
				SocketDescriptor socket;
				unsigned sendtimeout;
				unsigned recvtimeout;
				bool closed;
				bool reuseaddress;
		};


		bool UDPSocket::isClosed() const
		{
			return closed;
		}

		bool UDPSocket::isReusingAddress() const
		{
			return reuseaddress;
		}

		unsigned UDPSocket::getSendTimeout() const
		{
			return sendtimeout;
		}

		unsigned UDPSocket::getRecvTimeout() const
		{
			return recvtimeout;
		}

		SocketDescriptor UDPSocket::getDescriptor() const
		{
			return socket;
		}
	}
}

#endif
