#ifndef KRY_SYSTEM_FILESYSTEM_H
#define KRY_SYSTEM_FILESYSTEM_H

#include "Utilities\StringTypes.h"
#include <vector>

namespace kry
{
	namespace System
	{
		enum class FileMode : char
		{
			OVERWRITE,
			APPEND
		};

		enum class FileAccess : short
		{
			NONE = 0,
			READ = 1,
			WRITE = 2,
			EXECUTE = 4
		};
		FileAccess operator|(const FileAccess left, const FileAccess right);
		FileAccess operator&(const FileAccess left, const FileAccess right);

		bool fileExists(const Util::String& filename);
		bool pathExists(const Util::String& directory);

		void createFile(const Util::String& filename, bool overwrite = false, FileAccess access = FileAccess::READ | FileAccess::WRITE);
		void createPath(const Util::String& directory);
		void copyFile(const Util::String& source, const Util::String& destination);
		void copyPath(const Util::String& source, const Util::String& destination);
		void moveFile(const Util::String& source, const Util::String& destination, bool waitForCompletion);
		void movePath(const Util::String& source, const Util::String& destination);
		void deleteFile(const Util::String& filename);
		void deletePath(const Util::String& filename);

		void writeAllLines(const Util::String& filename, const Util::String& lines, char delimiter = '\n', FileMode mode = FileMode::OVERWRITE);
		void writeAllLines(const Util::String& filename, const std::vector<Util::String>& lines, FileMode mode = FileMode::OVERWRITE);
		std::vector<Util::String> readAllLines(const Util::String& filename, char delimiter = '\n');

		void writeAllBytes(const Util::String& filename, const char* buffer, unsigned bufferlength, FileMode mode = FileMode::OVERWRITE);
		void writeAllBytes(const Util::String& filename, const std::vector<char>& buffer, FileMode mode = FileMode::OVERWRITE);
		std::vector<char> readAllBytes(const Util::String& filename);

		std::vector<Util::String> getAllFiles(const Util::String& directory, bool shortened = false);
		std::vector<Util::String> getAllFiles(const Util::String& directory, const std::vector<Util::String>& extfilter, bool shortened = false);
		std::vector<Util::String> getAllPaths(const Util::String& directory, bool shortened = false);
	}
}

#endif
