#ifndef KRY_SYSTEM_WINDOW_H
#define KRY_SYSTEM_WINDOW_H

#include "System\WindowEvent.h"
#include "System\WindowTypes.h"
#include "Utilities\StringTypes.h"
#include <queue>
#include <functional>

namespace kry
{
	namespace System // raw input device
	{
		typedef std::function<long __stdcall(WindowHandle winhandle, unsigned event, unsigned wparam, long lparam)> WindowProcess;

		enum class WindowMode : char
		{
			WINDOWED,
			BORDERLESS,
			FULLSCREEN
		};

		class Window
		{
			public:
				Window(const Util::String& title, const Util::Vector2i& size);
				virtual ~Window();

				void setTitle(const Util::String& title);
				void setSize(const Util::Vector2i& size);
				void setPosition(const Util::Vector2i& position);
				Util::String getTitle() const;
				Util::Vector2i getSize() const; // returns the client size
				Util::Vector2i getPosition() const;

				bool peekEvent(WindowEvent& event);
				bool getEvent(WindowEvent& event);
				bool waitEvent(WindowEvent& event);

				void setMode(WindowMode mode);
				void setVsync(bool vsync) const;
				void clipCursor(bool clip) const;
				void hideCursor(bool hide) const;
				Util::Vector2i getCursorPos() const;
				void destroy();

				inline WindowHandle getHandle() const;
				inline DeviceContext getContext() const;
				inline GLContext getGLContext() const;
				inline const std::queue<WindowEvent>& getQueuedEvents() const;
				inline WindowMode getWindowMode() const;
				inline bool isDestroyed() const;

				static Util::Vector2i getScreenSize(Window* source = nullptr); // if source != nullptr, get the monitor info holding the window
				static unsigned getMonitorCount();
				static void messageBox(const Util::String& title, const Util::String& message, Window* owner = nullptr);

			protected:
				void hookWindowProcess(const WindowProcess& process);

				WindowHandle handle;
				DeviceContext context;
				GLContext glcontext;
				std::queue<WindowEvent> events;
				WindowProcess windowproc;
				WindowMode winmode;

			private:
				Window();
				Window(const Window& window);
				Window& operator=(const Window& window);
				void processEvent();

				bool destroyed;

				static WindowClass winclass;
				static unsigned wincount;
				static long __stdcall winProc(WindowHandle winhandle, unsigned event, unsigned wparam, long lparam);
		};


		WindowHandle Window::getHandle() const
		{
			return handle;
		}

		DeviceContext Window::getContext() const
		{
			return context;
		}

		GLContext Window::getGLContext() const
		{
			return glcontext;
		}

		const std::queue<WindowEvent>& Window::getQueuedEvents() const
		{
			return events;
		}

		WindowMode Window::getWindowMode() const
		{
			return winmode;
		}

		bool Window::isDestroyed() const
		{
			return destroyed;
		}
	}
}

#endif
