#ifndef KRY_UTILITIES_BASICSTRING_H
#define KRY_UTILITIES_BASICSTRING_H

#include <vector>
#include <ostream>
#include <istream>

/** #TODO(incomplete) may optimise by taking the capacity into account when using operations */

namespace kry
{
	namespace Util
	{
		template <typename CharPtr>
		inline void strcpy(CharPtr source, CharPtr dest, unsigned length);
		template <typename CharType>
		inline void strset(CharType* dest, CharType val, unsigned length);
		template <typename CharPtr>
		inline int strcmp(CharPtr left, CharPtr right, unsigned length);

		template <typename StringType>
		class BasicString;

		template <typename StringType>
		inline BasicString<StringType> operator+(const BasicString<StringType>& left, const BasicString<StringType>& right);
		/** #TODO(change) these are probably slower than they could be, considering we already know the length of the array, could just simply copy it over. */
		template <typename StringType, std::size_t TypeSize, typename UnConst = std::remove_const<StringType>::type>
		inline BasicString<UnConst> operator+(const BasicString<UnConst>& left, StringType(&right)[TypeSize]); 
		template <typename StringType, std::size_t TypeSize, typename UnConst = std::remove_const<StringType>::type>
		inline BasicString<UnConst> operator+(StringType(&left)[TypeSize], const BasicString<UnConst>& right);
		template <typename StringType>
		inline BasicString<StringType> operator+(const BasicString<StringType>& left, StringType right);
		template <typename StringType>
		inline BasicString<StringType> operator+(StringType left, const BasicString<StringType>& right);
		template <typename StringType>
		inline bool operator==(const BasicString<StringType>& left, const BasicString<StringType>& right);
		template <typename StringType, std::size_t TypeSize, typename UnConst = std::remove_const<StringType>::type>
		inline bool operator==(const BasicString<UnConst>& left, StringType(&right)[TypeSize]);
		template <typename StringType, std::size_t TypeSize, typename UnConst = std::remove_const<StringType>::type>
		inline bool operator==(StringType(&left)[TypeSize], const BasicString<UnConst>& right);
		template <typename StringType>
		inline bool operator!=(const BasicString<StringType>& left, const BasicString<StringType>& right);
		template <typename StringType, std::size_t TypeSize, typename UnConst = std::remove_const<StringType>::type>
		inline bool operator!=(const BasicString<UnConst>& left, StringType(&right)[TypeSize]);
		template <typename StringType, std::size_t TypeSize, typename UnConst = std::remove_const<StringType>::type>
		inline bool operator!=(StringType(&left)[TypeSize], const BasicString<UnConst>& right);
		template <typename StringType>
		inline bool operator>=(const BasicString<StringType>& left, const BasicString<StringType>& right);
		template <typename StringType, std::size_t TypeSize, typename UnConst = std::remove_const<StringType>::type>
		inline bool operator>=(const BasicString<UnConst>& left, StringType(&right)[TypeSize]);
		template <typename StringType, std::size_t TypeSize, typename UnConst = std::remove_const<StringType>::type>
		inline bool operator>=(StringType(&left)[TypeSize], const BasicString<UnConst>& right);
		template <typename StringType>
		inline bool operator<=(const BasicString<StringType>& left, const BasicString<StringType>& right);
		template <typename StringType, std::size_t TypeSize, typename UnConst = std::remove_const<StringType>::type>
		inline bool operator<=(const BasicString<UnConst>& left, StringType(&right)[TypeSize]);
		template <typename StringType, std::size_t TypeSize, typename UnConst = std::remove_const<StringType>::type>
		inline bool operator<=(StringType(&left)[TypeSize], const BasicString<UnConst>& right);
		template <typename StringType>
		inline bool operator>(const BasicString<StringType>& left, const BasicString<StringType>& right);
		template <typename StringType, std::size_t TypeSize, typename UnConst = std::remove_const<StringType>::type>
		inline bool operator>(const BasicString<UnConst>& left, StringType(&right)[TypeSize]);
		template <typename StringType, std::size_t TypeSize, typename UnConst = std::remove_const<StringType>::type>
		inline bool operator>(StringType(&left)[TypeSize], const BasicString<UnConst>& right);
		template <typename StringType>
		inline bool operator<(const BasicString<StringType>& left, const BasicString<StringType>& right);
		template <typename StringType, std::size_t TypeSize, typename UnConst = std::remove_const<StringType>::type>
		inline bool operator<(const BasicString<UnConst>& left, StringType(&right)[TypeSize]);
		template <typename StringType, std::size_t TypeSize, typename UnConst = std::remove_const<StringType>::type>
		inline bool operator<(StringType(&left)[TypeSize], const BasicString<UnConst>& right);
		template <typename StringType>
		inline std::ostream& operator<<(std::ostream& left, const BasicString<StringType>& right);
		template <typename StringType>
		inline std::istream& operator>>(std::istream& left, BasicString<StringType>& right);

		template <typename StringType>
		class BasicString
		{
			public:
				BasicString(); // default
				template <typename LiteralType, std::size_t TypeSize>
				BasicString(LiteralType(&string)[TypeSize]); // literal copy
				BasicString(const StringType* string, unsigned length); // copy buffer
				BasicString(unsigned length, StringType fill); // fill
				explicit BasicString(unsigned capacity); // initialise capacity
				BasicString(const BasicString<StringType>& copy); // copy
				BasicString(BasicString<StringType>&& move); // move
				virtual ~BasicString();

				// finds
				long indexOf(const BasicString<StringType>& tofind, unsigned index = 0) const; // cast the result to an unsigned to get the actual index, if it's not -1
				long indexOf(StringType tofind, unsigned index = 0) const;
				long lastIndexOf(const BasicString<StringType>& tofind) const;
				long lastIndexOf(const BasicString<StringType>& tofind, unsigned index) const;
				long lastIndexOf(StringType tofind) const;
				long lastIndexOf(StringType tofind, unsigned index) const;
				BasicString<StringType> substring(unsigned index) const;
				BasicString<StringType> substring(unsigned index, unsigned length) const;
				BasicString<StringType> replace(const BasicString<StringType>& toreplace, const BasicString<StringType>& replacewith) const;
				BasicString<StringType> replace(StringType toreplace, StringType replacewith) const;
				// checks
				bool contains(const BasicString<StringType>& totest, unsigned index = 0) const;
				bool contains(StringType totest, unsigned index = 0) const;
				bool startsWith(const BasicString<StringType>& totest) const;
				bool endsWith(const BasicString<StringType>& totest) const;
				// extras
				std::vector<BasicString<StringType> > explode(const BasicString<StringType>& delim) const;
				std::vector<BasicString<StringType> > explode(StringType delim) const;
				BasicString<StringType> insert(const BasicString<StringType>& toinsert, unsigned index) const;
				BasicString<StringType> insert(StringType toinsert, unsigned index) const;
				BasicString<StringType> reverse() const;
				BasicString<StringType> resize(unsigned newlength) const;
				void shrink();
				void clear();

				// virtuals
				virtual BasicString<StringType> trim() const;

				// member operators
				BasicString<StringType>& operator=(const BasicString<StringType>& copy); /** #TODO(incomplete) create overloads for string literals here as well */
				BasicString<StringType>& operator=(BasicString<StringType>&& move);
				BasicString<StringType>& operator+=(const BasicString<StringType>& right);
				BasicString<StringType>& operator+=(StringType right);
				StringType& operator[](unsigned index) const;

				// getters
				inline const StringType* getData() const;
				inline unsigned getLength() const;
				inline unsigned getCapacity() const;
				inline bool isEmpty() const;

			protected:
				BasicString<StringType>(StringType* string, unsigned length, unsigned offset); // substring

				unsigned capacity;
				unsigned length;
				StringType* string;
				
#pragma warning(disable : 4396)
			friend BasicString<StringType> operator+ <> (const BasicString<StringType>& left, const BasicString<StringType>& right);
			friend BasicString<StringType> operator+ <> (const BasicString<StringType>& left, StringType right);
			friend BasicString<StringType> operator+ <> (StringType left, const BasicString<StringType>& right);
			friend bool operator== <> (const BasicString<StringType>& left, const BasicString<StringType>& right);
			friend bool operator!= <> (const BasicString<StringType>& left, const BasicString<StringType>& right);
			friend bool operator>= <> (const BasicString<StringType>& left, const BasicString<StringType>& right);
			friend bool operator<= <> (const BasicString<StringType>& left, const BasicString<StringType>& right);
			friend bool operator> <> (const BasicString<StringType>& left, const BasicString<StringType>& right);
			friend bool operator< <> (const BasicString<StringType>& left, const BasicString<StringType>& right);
			friend std::ostream& operator<< <> (std::ostream& left, const BasicString<StringType>& right);
			friend std::istream& operator>> <> (std::istream& left, BasicString<StringType>& right);
#pragma warning(default : 4396)
		};

		template<typename StringType, std::size_t TypeSize, typename UnConst = std::remove_const<StringType>::type>
		BasicString<UnConst> stringLiteral(StringType(&string)[TypeSize]); /** #TODO(incomplete) this should be a constexpr when it becomes available */
	}
}

#include "Utilities\Exception.h"
#include "StringConvert.h"

namespace kry
{
	namespace Util
	{
		template <typename CharPtr>
		void strcpy(CharPtr source, CharPtr dest, unsigned length)
		{
			while (length-- != 0)
			{
				*dest = *source;
				dest += 1;
				source += 1;
			}
		}

		template <typename CharType>
		void strset(CharType* dest, CharType val, unsigned length)
		{
			while (length-- != 0)
			{
				*dest = val;
				dest += 1;
			}
		}

		template <typename CharPtr>
		int strcmp(CharPtr left, CharPtr right, unsigned length)
		{
			for (unsigned i = 0; i < length; ++i)
			{
				if (*left == '\0' || *left != *right)
					return static_cast<int>(*left - *right);
				left += 1;
				right += 1;
			}

			return 0;
		}

		template <typename StringType>
		BasicString<StringType>::BasicString() : capacity(1), length(0), string(new StringType[capacity]{'\0'})
		{
		}

		template <typename StringType>
		template <typename LiteralType, std::size_t TypeSize>
		BasicString<StringType>::BasicString(LiteralType(&string)[TypeSize]) : capacity(TypeSize), length(TypeSize - 1), string(new StringType[capacity])
		{
			if (string == nullptr)
				throw Exception(String("Null pointer passed! [pointer=string]"), KRY_EXCEPTION_DATA);

			this->string[length] = '\0'; // better add the null byte, cant trust the user
			strcpy(const_cast<StringType*>(string), this->string, length);
		}

		template <typename StringType>
		BasicString<StringType>::BasicString(const StringType* string, unsigned length) : capacity(length + 1), length(length), string(new StringType[capacity])
		{
			if (string == nullptr)
				throw Exception(String("Null pointer passed! [pointer=string]"), KRY_EXCEPTION_DATA);

			this->string[length] = '\0'; // better add the null byte, cant trust the user
			strcpy(const_cast<StringType*>(string), this->string, length);
		}
		
		template <typename StringType>
		BasicString<StringType>::BasicString(unsigned length, StringType fill) : capacity(length + 1), length(length), string(new StringType[capacity]{fill})
		{
			string[length] = '\0';
		}
		
		template <typename StringType>
		BasicString<StringType>::BasicString(unsigned capacity) : capacity(capacity), length(capacity - 1), string(new StringType[capacity]{'\0'})
		{
		}
		
		template <typename StringType>
		BasicString<StringType>::BasicString(const BasicString<StringType>& copy) : capacity(copy.capacity), length(copy.length), string(new StringType[copy.capacity])
		{
			strcpy(copy.string, string, capacity);
		}
		
		template <typename StringType>
		BasicString<StringType>::BasicString(BasicString<StringType>&& move) : capacity(move.capacity), length(move.length), string(move.string)
		{
			move.string = nullptr;
		}
		
		template <typename StringType>
		BasicString<StringType>::~BasicString()
		{
			delete[] string;
		}

		template <typename StringType>
		long BasicString<StringType>::indexOf(const BasicString<StringType>& tofind, unsigned index) const
		{
			if (index + tofind.length > length)
				throw Exception("Index out of range! [index=" + toString(index) + ", length=" + toString(length) +
								", tofind::length=" + toString(tofind.length) + ']', KRY_EXCEPTION_DATA);

			while (index + tofind.length <= length)
			{
				if (strcmp(string + index, tofind.string, tofind.length) == 0)
					return static_cast<long>(index);
				++index;
			}
			return -1;
		}
		
		template <typename StringType>
		long BasicString<StringType>::indexOf(StringType tofind, unsigned index) const
		{
			if (index >= length)
				throw Exception("Index out of range! [index=" + toString(index) + ", length=" + toString(length) + ']', 
								KRY_EXCEPTION_DATA);

			while (index < length)
			{
				if (string[index] == tofind)
					return index;
				++index;
			}

			return -1;
		}
		
		template <typename StringType>
		long BasicString<StringType>::lastIndexOf(const BasicString<StringType>& tofind) const
		{
			return lastIndexOf(tofind, length - tofind.length);
		}
		
		template <typename StringType>
		long BasicString<StringType>::lastIndexOf(const BasicString<StringType>& tofind, unsigned index) const
		{
			if (index + tofind.length > length)
				throw Exception("Index out of range! [index=" + toString(index) + ", length=" + toString(length) + 
								", tofind::length=" + toString(tofind.length) + ']', KRY_EXCEPTION_DATA);

			do
			{
				if (strcmp(string + index, tofind.string, tofind.length) == 0)
					return index;
			} while (index-- != 0);

			return -1;
		}
		
		template <typename StringType>
		long BasicString<StringType>::lastIndexOf(StringType tofind) const
		{
			return lastIndexOf(tofind, length - 1);
		}
		
		template <typename StringType>
		long BasicString<StringType>::lastIndexOf(StringType tofind, unsigned index) const
		{
			if (index >= length)
				throw Exception("Index out of range! [index=" + toString(index) + ", length=" + toString(length) + ']', 
								KRY_EXCEPTION_DATA);
			
			do
			{
				if (string[index] == tofind)
					return index;
			} while (index-- != 0);

			return -1;
		}
		
		template <typename StringType>
		BasicString<StringType> BasicString<StringType>::substring(unsigned index) const
		{
			if (index >= length)
				throw Exception("Index out of range! [index=" + toString(index) + ", length=" + toString(length) + ']', 
								KRY_EXCEPTION_DATA);

			return BasicString<StringType>(string, length - index, index);
		}
		
		template <typename StringType>
		BasicString<StringType> BasicString<StringType>::substring(unsigned index, unsigned len) const
		{
			if (index + len > length)
				throw Exception("Index out of range! [index=" + toString(index) + ", length=" + toString(len) +
				", this::length=" + toString(length) + ']', KRY_EXCEPTION_DATA);

			return BasicString<StringType>(string, len, index);
		}
		
		template <typename StringType> /** #TODO(bug) does not work for empty strings, or strings with a length of 1, fix */
		BasicString<StringType> BasicString<StringType>::replace(const BasicString<StringType>& toreplace, const BasicString<StringType>& replacewith) const
		{
			BasicString<StringType> tmp;
			std::vector<BasicString<StringType>> cols(std::move(explode(toreplace)));

			auto size = cols.size();
			for (decltype(size) i = 0; i < size; ++i)
			{
				tmp += cols[i];
				if (i < size - 1)
					tmp += replacewith;
			}
			return tmp;
		}
		
		template <typename StringType>
		BasicString<StringType> BasicString<StringType>::replace(StringType toreplace, StringType replacewith) const
		{
			BasicString<StringType> tmp(*this);
			long index = -1;
			while (static_cast<unsigned>(index + 1) < length && (index = tmp.indexOf(toreplace, static_cast<unsigned>(index + 1))) != -1)
				tmp[index] = replacewith;
			return tmp;
		}
		
		template <typename StringType>
		bool BasicString<StringType>::contains(const BasicString<StringType>& totest, unsigned index) const // really simple find algo, could be sped up if need be
		{
			if (isEmpty())
				return false;
			return indexOf(totest, index) != -1;
		}
		
		template <typename StringType>
		bool BasicString<StringType>::contains(StringType totest, unsigned index) const
		{
			if (isEmpty())
				return false;
			return indexOf(totest, index) != -1;
		}
		
		template <typename StringType>
		bool BasicString<StringType>::startsWith(const BasicString<StringType>& totest) const
		{
			if (totest.length > length)
				return false;
			return strcmp(string, totest.string, totest.length) == 0;
		}
		
		template <typename StringType>
		bool BasicString<StringType>::endsWith(const BasicString<StringType>& totest) const
		{
			if (totest.length > length)
				return false;
			return strcmp(string + (length - totest.length), totest.string, totest.length) == 0;
		}
		
		template <typename StringType>
		std::vector<BasicString<StringType> > BasicString<StringType>::explode(const BasicString<StringType>& delim) const
		{
			std::vector<BasicString<StringType> > columns;

			if (!delim.isEmpty())
			{
				long long lastindex = 0;
				long long index = indexOf(delim);
				while (index != -1)
				{
					if (index != 0)
						columns.push_back(BasicString<StringType>(string + lastindex, static_cast<unsigned>(index - lastindex)));
					lastindex = index + delim.length;
					if (lastindex >= length)
						break;
					index = indexOf(delim, lastindex);
				}
				if (static_cast<unsigned long long>(lastindex) < length)
					columns.push_back(BasicString<StringType>(string + lastindex, static_cast<unsigned>(length - lastindex)));
			}
			else
				columns.push_back(BasicString<StringType>(*this));
			return columns;
		}
		
		template <typename StringType>
		std::vector<BasicString<StringType> > BasicString<StringType>::explode(StringType delim) const
		{
			std::vector<BasicString<StringType> > columns;

			long long lastindex = 0;
			long long index = indexOf(delim);
			while (index != -1)
			{
				if (index != 0)
					columns.push_back(BasicString<StringType>(string + lastindex, static_cast<unsigned>(index - lastindex)));
				lastindex = index + 1;
				if (lastindex == length)
					break;
				index = indexOf(delim, static_cast<unsigned>(lastindex));
			}
			if (static_cast<unsigned long long>(lastindex) < length)
				columns.push_back(BasicString<StringType>(string + lastindex, static_cast<unsigned>(length - lastindex)));
			return columns;
		}
		
		template <typename StringType>
		BasicString<StringType> BasicString<StringType>::insert(const BasicString<StringType>& toinsert, unsigned index) const
		{
			if (index > length)
				throw Exception("Index out of range! [index=" + toString(index) + ", length=" + toString(length) + ']', 
								KRY_EXCEPTION_DATA);
			if (index == 0)
				return toinsert + *this;
			else if (index == length)
				return *this + toinsert;

			BasicString<StringType> left(string, index);
			BasicString<StringType> right(string, length - index, index);
			return (left + toinsert) + right;
		}
		
		template <typename StringType>
		BasicString<StringType> BasicString<StringType>::insert(StringType toinsert, unsigned index) const
		{
			if (index > length)
				throw Exception("Index out of range! [index=" + toString(index) + ", length=" + toString(length) + ']', 
								KRY_EXCEPTION_DATA);
			if (index == 0)
				return toinsert + *this;
			else if (index == length)
				return *this + toinsert;

			BasicString<StringType> left(string, index);
			BasicString<StringType> right(string, length - index, index);
			return (left + toinsert) + right;
		}
		
		template <typename StringType>
		BasicString<StringType> BasicString<StringType>::reverse() const
		{
			BasicString<StringType> tmp(*this);
			unsigned len = length / 2;
			for (unsigned i = 0; i < len; ++i)
			{
				StringType c = tmp.string[i];
				tmp.string[i] = tmp.string[length - 1 - i];
				tmp.string[length - 1 - i] = c;
			}

			return tmp;
		}
		
		template <typename StringType>
		BasicString<StringType> BasicString<StringType>::trim() const
		{
			if (length == 0)
				return *this;
			else if (length == 1)
				return (string[0] < 33 || string[0] == 127) ? BasicString() : *this;
			
			unsigned lhs = 0;
			while ((string[lhs] < 33 || string[lhs] == 127) && lhs < length - 1)
				++lhs;
			unsigned rhs = length;
			while ((string[rhs - 1] < 33 || string[rhs - 1] == 127) && rhs > 0)
				--rhs;
			if (rhs == 0)
				return BasicString();
			return substring(lhs, (rhs - lhs));
		}
		
		template <typename StringType>
		BasicString<StringType> BasicString<StringType>::resize(unsigned newlength) const
		{
			if (newlength == length)
				return BasicString<StringType>(*this);
			else if (newlength < length)
				return BasicString<StringType>(string, newlength);

			BasicString<StringType> tmp(newlength, '\0');
			strcpy(string, tmp.string, length);
			return tmp;
		}
		
		template <typename StringType>
		void BasicString<StringType>::shrink()
		{
			if (capacity > length + 1)
			{
				BasicString<StringType> tmp(string, length);
				*this = std::move(tmp);
			}
		}
		
		template <typename StringType>
		void BasicString<StringType>::clear()
		{
			capacity = 1;
			delete[] string;
			string = new StringType[capacity]{'\0'};
			length = 0;
		}

		
		template <typename StringType>
		BasicString<StringType>& BasicString<StringType>::operator=(const BasicString<StringType>& copy)
		{
			if (this == &copy)
				return *this;

			if (capacity != copy.capacity)
			{
				delete[] string;
				string = new StringType[copy.capacity];
			}
			
			capacity = copy.capacity;
			strcpy(copy.string, string, capacity);
			length = copy.length;
			return *this;
		}
		
		template <typename StringType>
		BasicString<StringType>& BasicString<StringType>::operator=(BasicString<StringType>&& move)
		{
			if (this == &move)
				return *this;

			delete[] string;
			capacity = move.capacity;
			string = move.string;
			length = move.length;
			move.string = nullptr;
			return *this;
		}
		
		template <typename StringType>
		BasicString<StringType>& BasicString<StringType>::operator+=(const BasicString<StringType>& right)
		{
			if (length + right.length < capacity - 1)
			{
				strcpy(right.string, string + length, right.length);
				length += right.length;
				string[length] = '\0';
				return *this;
			}

			BasicString<StringType> tmp(std::move(*this + right));
			*this = std::move(tmp);
			return *this;
		}
		
		template <typename StringType>
		BasicString<StringType>& BasicString<StringType>::operator+=(StringType right)
		{
			if (length < capacity - 1)
			{
				string[length] = right;
				string[++length] = '\0';
				return *this;
			}

			BasicString<StringType> tmp(std::move(*this + right));
			*this = std::move(tmp);
			return *this;
		}
		
		template <typename StringType>
		StringType& BasicString<StringType>::operator[](unsigned index) const
		{
			if (index >= capacity)
				throw Exception("Index out of range! [index=" + toString(index) + ", capacity=" + toString(capacity) + ']', KRY_EXCEPTION_DATA);

			return string[index];
		}
		
		template <typename StringType>
		const StringType* BasicString<StringType>::getData() const
		{
			return string;
		}
		
		template <typename StringType>
		unsigned BasicString<StringType>::getLength() const
		{
			return length;
		}
		
		template <typename StringType>
		unsigned BasicString<StringType>::getCapacity() const
		{
			return capacity;
		}
		
		template <typename StringType>
		bool BasicString<StringType>::isEmpty() const
		{
			return length == 0;
		}
		
		template <typename StringType>
		BasicString<StringType>::BasicString(StringType* string, unsigned length, unsigned offset) : 
			capacity(length + 1), length(length), string(new StringType[capacity]{0})
		{
			strcpy(string + offset, this->string, length);
			this->string[length] = '\0';
		}

		
		template <typename StringType>
		BasicString<StringType> operator+(const BasicString<StringType>& left, const BasicString<StringType>& right)
		{
			BasicString<StringType> tmp(left.length + right.length, '\0');
			strcpy(left.string, tmp.string, left.length);
			strcpy(right.string, tmp.string + left.length, right.length);
			return tmp;
		}
		
		template <typename StringType, std::size_t TypeSize, typename UnConst>
		inline BasicString<UnConst> operator+(const BasicString<UnConst>& left, StringType(&right)[TypeSize])
		{
			return operator+(left, BasicString<UnConst>(right, TypeSize - 1));
		}

		template <typename StringType, std::size_t TypeSize, typename UnConst>
		inline BasicString<UnConst> operator+(StringType(&left)[TypeSize], const BasicString<UnConst>& right)
		{
			return operator+(BasicString<UnConst>(left, TypeSize - 1), right);
		}
		
		template <typename StringType>
		BasicString<StringType> operator+(const BasicString<StringType>& left, StringType right)
		{
			BasicString<StringType> tmp(left.length + 1, '\0');
			strcpy(left.string, tmp.string, left.length);
			tmp.string[left.length] = right;
			return tmp;
		}
		
		template <typename StringType>
		BasicString<StringType> operator+(StringType left, const BasicString<StringType>& right)
		{
			BasicString<StringType> tmp(right.length + 1, '\0');
			strcpy(right.string, tmp.string + 1, right.length);
			tmp.string[0] = left;
			return tmp;
		}
		
		template <typename StringType>
		bool operator==(const BasicString<StringType>& left, const BasicString<StringType>& right)
		{
			if (left.length != right.length)
				return false;
			return strcmp(left.string, right.string, left.length) == 0;
		}
		
		template <typename StringType, std::size_t TypeSize, typename UnConst>
		inline bool operator==(const BasicString<UnConst>& left, StringType(&right)[TypeSize])
		{
			return operator==(left, BasicString<UnConst>(right, TypeSize - 1));
		}

		template <typename StringType, std::size_t TypeSize, typename UnConst>
		inline bool operator==(StringType(&left)[TypeSize], const BasicString<UnConst>& right)
		{
			return operator==(BasicString<UnConst>(left, TypeSize - 1), right);
		}
		
		template <typename StringType>
		bool operator!=(const BasicString<StringType>& left, const BasicString<StringType>& right)
		{
			return !operator==(left, right);
		}
		
		template <typename StringType, std::size_t TypeSize, typename UnConst>
		inline bool operator!=(const BasicString<UnConst>& left, StringType(&right)[TypeSize])
		{
			return operator!=(left, BasicString<UnConst>(right, TypeSize - 1));
		}

		template <typename StringType, std::size_t TypeSize, typename UnConst>
		inline bool operator!=(StringType(&left)[TypeSize], const BasicString<UnConst>& right)
		{
			return operator!=(BasicString<UnConst>(left, TypeSize - 1), right);
		}
		
		template <typename StringType>
		bool operator>=(const BasicString<StringType>& left, const BasicString<StringType>& right)
		{
			return strcmp(left.string, right.string, std::max(left.length, right.length)) >= 0;
		}
		
		template <typename StringType, std::size_t TypeSize, typename UnConst>
		inline bool operator>=(const BasicString<UnConst>& left, StringType(&right)[TypeSize])
		{
			return operator>=(left, BasicString<UnConst>(right, TypeSize - 1));
		}

		template <typename StringType, std::size_t TypeSize, typename UnConst>
		inline bool operator>=(StringType(&left)[TypeSize], const BasicString<UnConst>& right)
		{
			return operator>=(BasicString<UnConst>(left, TypeSize - 1), right);
		}
		
		template <typename StringType>
		bool operator<=(const BasicString<StringType>& left, const BasicString<StringType>& right)
		{
			return strcmp(left.string, right.string, std::max(left.length, right.length)) <= 0;
		}
		
		template <typename StringType, std::size_t TypeSize, typename UnConst>
		inline bool operator<=(const BasicString<UnConst>& left, StringType(&right)[TypeSize])
		{
			return operator<=(left, BasicString<UnConst>(right, TypeSize - 1));
		}

		template <typename StringType, std::size_t TypeSize, typename UnConst>
		inline bool operator<=(StringType(&left)[TypeSize], const BasicString<UnConst>& right)
		{
			return operator<=(BasicString<UnConst>(left, TypeSize - 1), right);
		}
		
		template <typename StringType>
		bool operator>(const BasicString<StringType>& left, const BasicString<StringType>& right)
		{
			return strcmp(left.string, right.string, std::max(left.length, right.length)) > 0;
		}
		
		template <typename StringType, std::size_t TypeSize, typename UnConst>
		inline bool operator>(const BasicString<UnConst>& left, StringType(&right)[TypeSize])
		{
			return operator>(left, BasicString<UnConst>(right, TypeSize - 1));
		}

		template <typename StringType, std::size_t TypeSize, typename UnConst>
		inline bool operator>(StringType(&left)[TypeSize], const BasicString<UnConst>& right)
		{
			return operator>(BasicString<UnConst>(left, TypeSize - 1), right);
		}
		
		template <typename StringType>
		bool operator<(const BasicString<StringType>& left, const BasicString<StringType>& right)
		{
			return strcmp(left.string, right.string, std::max(left.length, right.length)) < 0;
		}
		
		template <typename StringType, std::size_t TypeSize, typename UnConst>
		inline bool operator<(const BasicString<UnConst>& left, StringType(&right)[TypeSize])
		{
			return operator<(left, BasicString<UnConst>(right, TypeSize - 1));
		}

		template <typename StringType, std::size_t TypeSize, typename UnConst>
		inline bool operator<(StringType(&left)[TypeSize], const BasicString<UnConst>& right)
		{
			return operator<(BasicString<UnConst>(left, TypeSize - 1), right);
		}
		
		template <typename StringType>
		std::ostream& operator<<(std::ostream& left, const BasicString<StringType>& right)
		{
			return left.write(static_cast<const char*>(right.getData()), right.length * sizeof(StringType));
		}
		
		template <typename StringType>
		std::istream& operator>>(std::istream& left, BasicString<StringType>& right)
		{
			long len = static_cast<long>(left.seekg(0, std::ios::end).tellg());
			if (len < 0)
				throw Exception("Error reading stream! [failbit=" + (left.fail() ? String("true") : String("false")) + 
								", badbit=" + (left.bad() ? String("true") : String("false")) + ']', KRY_EXCEPTION_DATA);
			right = BasicString<StringType>(static_cast<unsigned>(len), '\0');
			return left.seekg(0, std::ios::beg).read(right.string, right.length);
		}


		template<typename StringType, std::size_t TypeSize, typename UnConst>
		BasicString<UnConst> stringLiteral(StringType(&string)[TypeSize])
		{
			return BasicString<UnConst>(string, TypeSize - 1);
		}
	}
}
#endif
