#ifndef KRY_UTILITIES_MATH_H
#define KRY_UTILITIES_MATH_H

#include "Vector.h"

namespace kry
{
	namespace Util
	{
		#define KRY_PI 3.14159265f
		#define KRY_PIr 57.29577951f // (180/PI)
		#define KRY_PId 0.01745329f // (PI/180)

		float radiansToDegrees(const float Radians);
		float degreesToRadians(const float Degrees);
		int clampi(const int Value, const int Min, const int Max);
		float clampf(const float Value, const float Min, const float Max);
		float clampAngle(const float Value);
		bool lineLineInersect(const Vector4f& LineA, const Vector4f& LineB);
		bool boxPointIntersect(const Vector2f& Point, const Vector4f& Diagonals);
		bool boxPointIntersect(const Vector2f& Point, const Vector2f& BoxTopLeft, const Vector2f& BoxDimensions);
		bool boxBoxCollision(const Vector4f& DiagonalsA, const Vector4f& DiagonalsB);
		bool boxBoxCollision(const Vector2f& BoxTopLeftA, const Vector2f& BoxDimensionsA, const Vector2f& BoxTopLeftB, const Vector2f& BoxDimensionsB);
		bool lineBoxIntersect(const Vector4f& Line, const Vector2f& BoxTopLeft, const Vector2f& BoxDimensions);
		bool radialPointIntersect(const Vector2f& Point, const Vector4f& Ellipse);
		bool radialRadialIntersect(const Vector4f& EllipseA, const Vector4f& EllipseB);
		bool radialBoxIntersect(const Vector4f& Ellipse, const Vector2f& BoxTopLeft, const Vector2f& BoxDimensions);
		float ellipseAngleRadius(const Vector2f& Radius, float Angle);
		Vector2f trajectory(const Vector2f& Position, const float Angle, const float Distance);
		Vector3f trajectory(const Vector3f& Position, const Vector3f& Direction, const float Distance);
		float pointDirection(const Vector2f& PointA, const Vector2f& PointB);
		Vector3f pointDirection(const Vector3f& PointA, const Vector3f& PointB);
		float pointDistance(const Vector2f& PointA, const Vector2f& PointB);
		float pointDistance(const Vector3f& PointA, const Vector3f& PointB);
		Vector3f angleToDirection(const Vector3f& Angles);
		bool angleInCone(const float Angle, const float ConeLeft, const float ConeRight);
		Vector2f isoToOrtho(const Vector2f& Coord);
		Vector2f orthoToIso(const Vector2f& Coord);
		Vector4f absBox(Vector4f Diagonals);
		Vector4f boxToDiagonals(const Vector2f& Position, const Vector2f& Dimensions);
		Vector3f normalize(Vector3f Vector);
	}
}

#endif
