#include "Audio\DeviceContext.h"
#include "Utilities\Exception.h"
#include "OpenAL\alc.h"

namespace kry
{
	namespace Audio
	{
		std::unique_ptr<DeviceContext> DeviceContext::single(nullptr);

		DeviceContext::~DeviceContext()
		{
			if (context != nullptr)
			{
				alcMakeContextCurrent(nullptr); /** #TODO(bug) there's a crash here somehow */
				alcDestroyContext(context);
				context = nullptr;
			}
			if (device != nullptr)
			{
				alcCloseDevice(device);
				device = nullptr;
			}
		}

		const DeviceContext& DeviceContext::createDeviceContext(const Util::String& devicename)
		{
			if (!single)
				single.reset(new DeviceContext(devicename));
			return *single;
		}

		DeviceContext::DeviceContext(const Util::String& devicename)
		{
			device = alcOpenDevice(devicename.isEmpty() ? nullptr : devicename.getData());
			if (device == nullptr)
				throw Util::Exception("Failed to open audio device! [devicename=" + devicename + ']', KRY_EXCEPTION_DATA);
			context = alcCreateContext(device, nullptr);
			if (context == nullptr)
				throw Util::Exception("Failed to create audio context!", KRY_EXCEPTION_DATA);
			alcMakeContextCurrent(context);
		}

		DeviceContext::DeviceContext(const DeviceContext& device)
		{
		}

		DeviceContext& DeviceContext::operator=(const DeviceContext& device)
		{
			return *this;
		}
	}
}
