#include "Graphics\Texture.h"
#include "Utilities\String.h"
#include "Utilities\Exception.h"
#include <Windows.h>
#include <GL\glew.h>
#include <GL\gl.h>
#include <GL\glu.h>

namespace kry
{
	namespace Graphics
	{
		Texture::Texture()
		{
			textureID = 0;
			isMipMapped = false;
			life = new unsigned int(1);
		}

		Texture::~Texture()
		{
			if (life != nullptr)
			{
				if (--*life == 0)
				{
					free();
					delete life;
					life = nullptr;
				}
			}
		}

		Texture::Texture(const Texture& From)
		{
			textureID = From.textureID;
			dimensions = From.dimensions;
			isMipMapped = From.isMipMapped;
			life = From.life;
			++*life;
		}

		Texture& Texture::operator=(const Texture& From)
		{
			if (&From != this)
			{
				if (life != nullptr)
				{
					if (--*life == 0)
					{
						free();
						delete life;
						life = nullptr;
					}
				}

				textureID = From.textureID;
				dimensions = From.dimensions;
				isMipMapped = From.isMipMapped;
				life = From.life;
				++*life;
			}

			return *this;
		}

		void Texture::loadRGBA(std::vector<unsigned char>& RGBA, const Util::Vector2i& Dimensions, const bool MipMap)
		{
			if (RGBA.size() <= 0)
			{
				throw Util::Exception("Failed to load RGBA, empty vector!", KRY_EXCEPTION_DATA);
			}

			free();
			dimensions = Dimensions;
			isMipMapped = MipMap;
			glGenTextures(1, &textureID);
			glBindTexture(GL_TEXTURE_2D, textureID);
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);

			if (MipMap)
				gluBuild2DMipmaps(GL_TEXTURE_2D, GL_RGBA, Dimensions[0], Dimensions[1], GL_RGBA, GL_UNSIGNED_BYTE, &RGBA[0]);
			else
				glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, Dimensions[0], Dimensions[1], 0, GL_RGBA, GL_UNSIGNED_BYTE, &RGBA[0]);

			glBindTexture(GL_TEXTURE_2D, 0);
			glFinish();
		}

		void Texture::loadRGB(std::vector<unsigned char>& RGB, const Util::Vector2i& Dimensions, const bool MipMap)
		{
			if (RGB.size() <= 0)
			{
				throw Util::Exception("Failed to load RGB, empty vector!", KRY_EXCEPTION_DATA);
			}

			free();
			dimensions = Dimensions;
			isMipMapped = MipMap;
			glGenTextures(1, &textureID);
			glBindTexture(GL_TEXTURE_2D, textureID);
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);

			if (MipMap)
				gluBuild2DMipmaps(GL_TEXTURE_2D, GL_RGB, Dimensions[0], Dimensions[1], GL_RGB, GL_UNSIGNED_BYTE, &RGB[0]);
			else
				glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, Dimensions[0], Dimensions[1], 0, GL_RGB, GL_UNSIGNED_BYTE, &RGB[0]);

			glBindTexture(GL_TEXTURE_2D, 0);
			glFinish();
		}

		unsigned int Texture::getID() const
		{
			return textureID;
		}

		Util::Vector2i Texture::getDimensions() const
		{
			return dimensions;
		}

		void Texture::free()
		{
			if (textureID != 0)
				glDeleteTextures(1, &textureID);

			textureID = 0;
		}
	}
}