#include "Media\ImageFactory.h"
#include "Utilities\Exception.h"
#include "Utilities\String.h"
#include "Utilities\StringConvert.h"
#include "Utilities\BasicVector.h"
#define STB_IMAGE_IMPLEMENTATION
#include "..\libs\include\stb_image\stb_image.h"
#include <string>

namespace kry
{
	namespace Media
	{
		Graphics::Texture imageFileToTexture(const Util::String& Filepath, const bool MipMap)
		{
			int width, height, comp;
			Graphics::Texture texture;
			std::vector<unsigned char> rgba;
			unsigned char* buffer = stbi_load(Filepath.getData(), &width, &height, &comp, 0);

			if (buffer == nullptr || buffer == 0 || width < 0 || height < 0)
			{
				throw Util::Exception("Image decoder error! [file=" + Filepath + ", error=" + Util::String(stbi__g_failure_reason, strlen(stbi__g_failure_reason)) + ']', KRY_EXCEPTION_DATA);
			}

			if (comp > 0 && comp < 4)
			{
				buffer = stbi__convert_format(buffer, comp, 4, width, height);

				if (buffer == nullptr || buffer == 0 || width < 0 || height < 0)
				{
					throw Util::Exception("Image decoder error! [file=" + Filepath + ", error=" + Util::String(stbi__g_failure_reason, strlen(stbi__g_failure_reason)) + ']', KRY_EXCEPTION_DATA);
				}
			}
			else if (comp != 4)
			{
				throw Util::Exception("Cannot handle components required! [file=" + Filepath + ", components=" + Util::toString(comp) + ']', KRY_EXCEPTION_DATA);
			}

			rgba.insert(rgba.begin(), buffer, buffer + (sizeof(unsigned char) * 4 * width * height));
			texture.loadRGBA(rgba, Util::Vector2i(width, height), MipMap);
			stbi_image_free(buffer);

			return texture;
		}

		Graphics::Texture imageDataToTexture(const std::vector<unsigned char>& Data, const bool MipMap)
		{
			int width, height, comp;
			Graphics::Texture texture;
			std::vector<unsigned char> rgba;
			unsigned char* buffer = stbi_load_from_memory(const_cast<unsigned char*>(&Data[0]), Data.size(), &width, &height, &comp, 0);

			if (buffer == nullptr || buffer == 0 || width < 0 || height < 0)
			{
				throw Util::Exception("Image decoder error! [error=" + Util::String(stbi__g_failure_reason, strlen(stbi__g_failure_reason)) + ']', KRY_EXCEPTION_DATA);
			}

			if (comp > 0 && comp < 4)
			{
				buffer = stbi__convert_format(buffer, comp, 4, width, height);

				if (buffer == nullptr || buffer == 0 || width < 0 || height < 0)
				{
					throw Util::Exception("Image decoder error! [error=" + Util::String(stbi__g_failure_reason, strlen(stbi__g_failure_reason)) + ']', KRY_EXCEPTION_DATA);
				}
			}
			else if (comp != 4)
			{
				throw Util::Exception("Cannot handle components required! [components=" + Util::toString(comp) + ']', KRY_EXCEPTION_DATA);
			}

			rgba.insert(rgba.begin(), buffer, buffer + (sizeof(unsigned char) * 4 * width * height));
			texture.loadRGBA(rgba, Util::Vector2i(width, height), MipMap);
			stbi_image_free(buffer);

			return texture;
		}

		std::vector<unsigned char> imageFileToRGBA(const Util::String& Filepath, Util::Vector2i& Dimensions)
		{
			int width, height, comp;
			Graphics::Texture texture;
			std::vector<unsigned char> rgba;
			unsigned char* buffer = stbi_load(Filepath.getData(), &width, &height, &comp, 0);

			if (buffer == nullptr || buffer == 0 || width < 0 || height < 0)
			{
				throw Util::Exception("Image decoder error! [file=" + Filepath + ", error=" + Util::String(stbi__g_failure_reason, strlen(stbi__g_failure_reason)) + ']', KRY_EXCEPTION_DATA);
			}

			if (comp > 0 && comp < 4)
			{
				buffer = stbi__convert_format(buffer, comp, 4, width, height);

				if (buffer == nullptr || buffer == 0 || width < 0 || height < 0)
				{
					throw Util::Exception("Image decoder error! [file=" + Filepath + ", error=" + Util::String(stbi__g_failure_reason, strlen(stbi__g_failure_reason)) + ']', KRY_EXCEPTION_DATA);
				}
			}
			else if (comp != 4)
			{
				throw Util::Exception("Cannot handle components required! [file=" + Filepath + ", components=" + Util::toString(comp) + ']', KRY_EXCEPTION_DATA);
			}

			rgba.insert(rgba.begin(), buffer, buffer + (sizeof(unsigned char) * 4 * width * height));
			stbi_image_free(buffer);

			Dimensions = Util::Vector2i(width, height);

			return rgba;
		}

		std::vector<unsigned char> imageDataToRGBA(const std::vector<unsigned char>& Data, Util::Vector2i& Dimensions)
		{
			int width, height, comp;
			Graphics::Texture texture;
			std::vector<unsigned char> rgba;
			unsigned char* buffer = stbi_load_from_memory(const_cast<unsigned char*>(&Data[0]), Data.size(), &width, &height, &comp, 0);

			if (buffer == nullptr || buffer == 0 || width < 0 || height < 0)
			{
				throw Util::Exception("Image decoder error! [error=" + Util::String(stbi__g_failure_reason, strlen(stbi__g_failure_reason)) + ']', KRY_EXCEPTION_DATA);
			}

			if (comp > 0 && comp < 4)
			{
				buffer = stbi__convert_format(buffer, comp, 4, width, height);

				if (buffer == nullptr || buffer == 0 || width < 0 || height < 0)
				{
					throw Util::Exception("Image decoder error! [error=" + Util::String(stbi__g_failure_reason, strlen(stbi__g_failure_reason)) + ']', KRY_EXCEPTION_DATA);
				}
			}
			else if (comp != 4)
			{
				throw Util::Exception("Cannot handle components required! [components=" + Util::toString(comp) + ']', KRY_EXCEPTION_DATA);
			}

			rgba.insert(rgba.begin(), buffer, buffer + (sizeof(unsigned char) * 4 * width * height));
			stbi_image_free(buffer);

			Dimensions = Util::Vector2i(width, height);

			return rgba;
		}
	}
}
