#include "Network\NetworkDevice.h"
#include "Utilities\Exception.h"
#include "Utilities\ErrorMessage.h"
#include "Utilities\Logger.h"
#include <Windows.h>

namespace kry
{
	namespace Network
	{
		std::unique_ptr<NetworkDevice> NetworkDevice::single(nullptr);

		NetworkDevice::~NetworkDevice()
		{
			try
			{
				stopNetworking();
			}
			catch (const Util::Exception& e)
			{
				Util::Logger::log(e);
			}
		}

		void NetworkDevice::startNetworking()
		{
			if (started)
				return;

			unsigned verHigh = 2, verLow = 2;
			WSADATA wsaData;
			int err;
			if ((err = WSAStartup(MAKEWORD(verHigh, verLow), &wsaData)) != 0)
				throw Util::Exception("Failed to start network device! " + Util::parseWinError(err) + " [version=" + Util::toString(verHigh) + '.' + 
					Util::toString(verLow) + ']', KRY_EXCEPTION_DATA);
			majorversion = wsaData.wVersion;
			minorversion = wsaData.wHighVersion;
			maxsockets = wsaData.iMaxSockets;
			maxudpsize = wsaData.iMaxUdpDg;
			description = Util::String(wsaData.szDescription).trim();
			systemstatus = Util::String(wsaData.szSystemStatus).trim();

			started = true;
		}

		void NetworkDevice::stopNetworking()
		{
			if (!started)
				return;

			int err;
			if ((err = WSACleanup()) != 0)
				throw Util::Exception("Failed to stop network device! " + Util::parseWinError(err), KRY_EXCEPTION_DATA);

			started = false;
		}


		NetworkDevice* NetworkDevice::getDevice()
		{
			if (!single)
				single.reset(new NetworkDevice);
			return single.get();
		}


		NetworkDevice::NetworkDevice() : started(false), majorversion(0u), minorversion(0u), maxsockets(0u), maxudpsize(0u) 
		{
		}

		NetworkDevice::NetworkDevice(const NetworkDevice& device)
		{
		}

		NetworkDevice& NetworkDevice::operator=(const NetworkDevice& device)
		{
			return *this;
		}
	}
}
