#include "Process\Thread.h"
#include "Utilities\String.h"
#include "Utilities\Exception.h"
#include <Windows.h>
#include <process.h>

namespace kry
{
	namespace Process
	{
		unsigned int Thread::entry(void* usrData)
		{
			Thread* thread = reinterpret_cast<Thread*>(usrData);
			
			if (thread->procedure != nullptr)
				thread->procedure(); 

			return 0;
		}

		Thread::Thread()
		{
			handle = 0;
			procedure = nullptr;
			id = 0;
		}

		Thread::Thread(const std::function<void()> Procedure)
		{
			procedure = nullptr;
			handle = 0;
			id = 0;

			if (Procedure != nullptr)
			{
				procedure = Procedure;
				handle = reinterpret_cast<HANDLE>(_beginthreadex(0, 0, Thread::entry, reinterpret_cast<void*>(this), 0, &id));
			}
		}

		Thread::Thread(const Thread& From)
		{
			throw Util::Exception("Cannot copy threads!", KRY_EXCEPTION_DATA);
		}

		Thread& Thread::operator=(const Thread& From)
		{
			throw Util::Exception("Cannot copy threads!", KRY_EXCEPTION_DATA);
		}

		Thread::~Thread()
		{
			if (running())
			{
				throw Util::Exception("Thread prematurely lost from scope!", KRY_EXCEPTION_DATA);
			}

			close();
		}

		void Thread::join()
		{
			if (handle)
				WaitForSingleObject(handle, INFINITE);
		}

		void Thread::pause()
		{
			if (handle)
				SuspendThread(handle);
		}

		void Thread::resume()
		{
			if (handle)
				ResumeThread(handle);
		}

		void Thread::realtime()
		{
			if (handle)
				SetThreadPriority(handle, THREAD_PRIORITY_TIME_CRITICAL);
		}

		void Thread::close()
		{
			if (handle)
			{
				if (running())
					TerminateThread(handle, 0);

				CloseHandle(handle);
				handle = 0;
			}
		}

		bool Thread::running() const
		{
			if (handle)
				return (WaitForSingleObject(handle, 0) != WAIT_OBJECT_0);

			return false;
		}

	}
}