#include "Utilities\MD5.h"

namespace kry
{
	namespace Util
	{
		struct MD5Instance
		{
			unsigned char buffer[64];
			unsigned int count[2];
			unsigned int state[4];
			unsigned int digest[16];

			MD5Instance()
			{
				count[0] = 0;
				count[1] = 0;
				state[0] = 0x67452301;
				state[1] = 0xefcdab89;
				state[2] = 0x98badcfe;
				state[3] = 0x10325476;
			}
		};

		inline unsigned int rotate(unsigned int x, int n)
		{
			return (x << n) | (x >> (32 - n));
		}

		void transform(MD5Instance& MD5, const unsigned char* block)
		{
			unsigned int a = MD5.state[0];
			unsigned int b = MD5.state[1];
			unsigned int c = MD5.state[2];
			unsigned int d = MD5.state[3];
			unsigned int x[16];
			memcpy(x, block, sizeof(x));

			// Round 1
			a = rotate(a + (b&c | ~b&d) + x[0] + 0xd76aa478, 7) + b;
			d = rotate(d + (a&b | ~a&c) + x[1] + 0xe8c7b756, 12) + a;
			c = rotate(c + (d&a | ~d&b) + x[2] + 0x242070db, 17) + d;
			b = rotate(b + (c&d | ~c&a) + x[3] + 0xc1bdceee, 22) + c;
			a = rotate(a + (b&c | ~b&d) + x[4] + 0xf57c0faf, 7) + b;
			d = rotate(d + (a&b | ~a&c) + x[5] + 0x4787c62a, 12) + a;
			c = rotate(c + (d&a | ~d&b) + x[6] + 0xa8304613, 17) + d;
			b = rotate(b + (c&d | ~c&a) + x[7] + 0xfd469501, 22) + c;
			a = rotate(a + (b&c | ~b&d) + x[8] + 0x698098d8, 7) + b;
			d = rotate(d + (a&b | ~a&c) + x[9] + 0x8b44f7af, 12) + a;
			c = rotate(c + (d&a | ~d&b) + x[10] + 0xffff5bb1, 17) + d;
			b = rotate(b + (c&d | ~c&a) + x[11] + 0x895cd7be, 22) + c;
			a = rotate(a + (b&c | ~b&d) + x[12] + 0x6b901122, 7) + b;
			d = rotate(d + (a&b | ~a&c) + x[13] + 0xfd987193, 12) + a;
			c = rotate(c + (d&a | ~d&b) + x[14] + 0xa679438e, 17) + d;
			b = rotate(b + (c&d | ~c&a) + x[15] + 0x49b40821, 22) + c;

			// Round 2
			a = rotate(a + (b&d | c&~d) + x[1] + 0xf61e2562, 5) + b;
			d = rotate(d + (a&c | b&~c) + x[6] + 0xc040b340, 9) + a;
			c = rotate(c + (d&b | a&~b) + x[11] + 0x265e5a51, 14) + d;
			b = rotate(b + (c&a | d&~a) + x[0] + 0xe9b6c7aa, 20) + c;
			a = rotate(a + (b&d | c&~d) + x[5] + 0xd62f105d, 5) + b;
			d = rotate(d + (a&c | b&~c) + x[10] + 0x2441453, 9) + a;
			c = rotate(c + (d&b | a&~b) + x[15] + 0xd8a1e681, 14) + d;
			b = rotate(b + (c&a | d&~a) + x[4] + 0xe7d3fbc8, 20) + c;
			a = rotate(a + (b&d | c&~d) + x[9] + 0x21e1cde6, 5) + b;
			d = rotate(d + (a&c | b&~c) + x[14] + 0xc33707d6, 9) + a;
			c = rotate(c + (d&b | a&~b) + x[3] + 0xf4d50d87, 14) + d;
			b = rotate(b + (c&a | d&~a) + x[8] + 0x455a14ed, 20) + c;
			a = rotate(a + (b&d | c&~d) + x[13] + 0xa9e3e905, 5) + b;
			d = rotate(d + (a&c | b&~c) + x[2] + 0xfcefa3f8, 9) + a;
			c = rotate(c + (d&b | a&~b) + x[7] + 0x676f02d9, 14) + d;
			b = rotate(b + (c&a | d&~a) + x[12] + 0x8d2a4c8a, 20) + c;

			// Round 3
			a = rotate(a + (b^c^d) + x[5] + 0xfffa3942, 4) + b;
			d = rotate(d + (a^b^c) + x[8] + 0x8771f681, 11) + a;
			c = rotate(c + (d^a^b) + x[11] + 0x6d9d6122, 16) + d;
			b = rotate(b + (c^d^a) + x[14] + 0xfde5380c, 23) + c;
			a = rotate(a + (b^c^d) + x[1] + 0xa4beea44, 4) + b;
			d = rotate(d + (a^b^c) + x[4] + 0x4bdecfa9, 11) + a;
			c = rotate(c + (d^a^b) + x[7] + 0xf6bb4b60, 16) + d;
			b = rotate(b + (c^d^a) + x[10] + 0xbebfbc70, 23) + c;
			a = rotate(a + (b^c^d) + x[13] + 0x289b7ec6, 4) + b;
			d = rotate(d + (a^b^c) + x[0] + 0xeaa127fa, 11) + a;
			c = rotate(c + (d^a^b) + x[3] + 0xd4ef3085, 16) + d;
			b = rotate(b + (c^d^a) + x[6] + 0x4881d05, 23) + c;
			a = rotate(a + (b^c^d) + x[9] + 0xd9d4d039, 4) + b;
			d = rotate(d + (a^b^c) + x[12] + 0xe6db99e5, 11) + a;
			c = rotate(c + (d^a^b) + x[15] + 0x1fa27cf8, 16) + d;
			b = rotate(b + (c^d^a) + x[2] + 0xc4ac5665, 23) + c;

			// Round 4
			a = rotate(a + (c ^ (b | ~d)) + x[0] + 0xf4292244, 6) + b;
			d = rotate(d + (b ^ (a | ~c)) + x[7] + 0x432aff97, 10) + a;
			c = rotate(c + (a ^ (d | ~b)) + x[14] + 0xab9423a7, 15) + d;
			b = rotate(b + (d ^ (c | ~a)) + x[5] + 0xfc93a039, 21) + c;
			a = rotate(a + (c ^ (b | ~d)) + x[12] + 0x655b59c3, 6) + b;
			d = rotate(d + (b ^ (a | ~c)) + x[3] + 0x8f0ccc92, 10) + a;
			c = rotate(c + (a ^ (d | ~b)) + x[10] + 0xffeff47d, 15) + d;
			b = rotate(b + (d ^ (c | ~a)) + x[1] + 0x85845dd1, 21) + c;
			a = rotate(a + (c ^ (b | ~d)) + x[8] + 0x6fa87e4f, 6) + b;
			d = rotate(d + (b ^ (a | ~c)) + x[15] + 0xfe2ce6e0, 10) + a;
			c = rotate(c + (a ^ (d | ~b)) + x[6] + 0xa3014314, 15) + d;
			b = rotate(b + (d ^ (c | ~a)) + x[13] + 0x4e0811a1, 21) + c;
			a = rotate(a + (c ^ (b | ~d)) + x[4] + 0xf7537e82, 6) + b;
			d = rotate(d + (b ^ (a | ~c)) + x[11] + 0xbd3af235, 10) + a;
			c = rotate(c + (a ^ (d | ~b)) + x[2] + 0x2ad7d2bb, 15) + d;
			b = rotate(b + (d ^ (c | ~a)) + x[9] + 0xeb86d391, 21) + c;

			MD5.state[0] += a;
			MD5.state[1] += b;
			MD5.state[2] += c;
			MD5.state[3] += d;

			memset(x, 0, sizeof(x));
		}

		void update(MD5Instance& MD5, const unsigned char* Input, size_t Length)
		{
			size_t index = MD5.count[0] / 8 % 64;

			if ((MD5.count[0] += (Length << 3)) < (Length << 3))
			{
				++MD5.count[1];
			}

			MD5.count[1] += (Length >> 29);
			size_t firstPart = 64 - index;
			size_t i = 0;

			if (Length >= firstPart)
			{
				memcpy(&MD5.buffer[index], Input, firstPart);
				transform(MD5, MD5.buffer);

				for (i = firstPart; i + 64 <= Length; i += 64)
					transform(MD5, &Input[i]);

				index = 0;
			}

			memcpy(&MD5.buffer[index], &Input[i], Length - i);
		}

		void finalize(MD5Instance& MD5)
		{
			static unsigned char padding[64] = {
				0x80, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
				0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
				0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
			};
			unsigned char bits[8];
			bits[0] = MD5.count[0] & 0xff;
			bits[1] = (MD5.count[0] >> 8) & 0xff;
			bits[2] = (MD5.count[0] >> 16) & 0xff;
			bits[3] = (MD5.count[0] >> 24) & 0xff;
			bits[4] = MD5.count[1] & 0xff;
			bits[5] = (MD5.count[1] >> 8) & 0xff;
			bits[6] = (MD5.count[1] >> 16) & 0xff;
			bits[7] = (MD5.count[1] >> 24) & 0xff;
			size_t index = MD5.count[0] / 8 % 64;
			size_t padLength = (index < 56) ? (56 - index) : (120 - index);
			update(MD5, padding, padLength);
			update(MD5, bits, 8);
			MD5.digest[0] = MD5.state[0] & 0xff;
			MD5.digest[1] = (MD5.state[0] >> 8) & 0xff;
			MD5.digest[2] = (MD5.state[0] >> 16) & 0xff;
			MD5.digest[3] = (MD5.state[0] >> 24) & 0xff;
			MD5.digest[4] = MD5.state[1] & 0xff;
			MD5.digest[5] = (MD5.state[1] >> 8) & 0xff;
			MD5.digest[6] = (MD5.state[1] >> 16) & 0xff;
			MD5.digest[7] = (MD5.state[1] >> 24) & 0xff;
			MD5.digest[8] = MD5.state[2] & 0xff;
			MD5.digest[9] = (MD5.state[2] >> 8) & 0xff;
			MD5.digest[10] = (MD5.state[2] >> 16) & 0xff;
			MD5.digest[11] = (MD5.state[2] >> 24) & 0xff;
			MD5.digest[12] = MD5.state[3] & 0xff;
			MD5.digest[13] = (MD5.state[3] >> 8) & 0xff;
			MD5.digest[14] = (MD5.state[3] >> 16) & 0xff;
			MD5.digest[15] = (MD5.state[3] >> 24) & 0xff;
			memset(MD5.buffer, 0, sizeof(MD5.buffer));
			memset(MD5.count, 0, sizeof(MD5.count));
		}

		String stringToMD5(const String& Data)
		{
			MD5Instance MD5;
			update(MD5, reinterpret_cast<const unsigned char*>(Data.getData()), Data.getLength());
			finalize(MD5);

			String md5String;
			for (int i = 0; i < 16; ++i)
				md5String += toString(MD5.digest[i], IntegralBase::HEXADECIMAL);
			return md5String;
		}

		String dataToMD5(const std::vector<unsigned char>& Data)
		{
			MD5Instance MD5;
			update(MD5, Data.data(), Data.size());
			finalize(MD5);

			String md5String;
			for (int i = 0; i < 16; ++i)
				md5String += toString(MD5.digest[i], IntegralBase::HEXADECIMAL);
			return md5String;
		}

		String fileToMD5(const String& Filepath)
		{
			MD5Instance MD5;
			FILE *fp = 0;
			fopen_s(&fp, Filepath.getData(), "rb");

			if (!fp)
			{
				throw Util::Exception("MD5 error, could not open file! [file=" + Filepath + ']', KRY_EXCEPTION_DATA);
			}
			else
			{
				unsigned char buffer[4096];
				fseek(fp, 0, SEEK_END);
				size_t fileSize = ftell(fp);
				rewind(fp);
				size_t pos = 0;

				while (pos < fileSize)
				{
					size_t length = fread(reinterpret_cast<void*>(&buffer[0]), sizeof(unsigned char), sizeof(buffer), fp);
					std::vector<unsigned char> fileBuffer;
					fileBuffer.insert(fileBuffer.end(), buffer, buffer + length);
					pos += length;

					update(MD5, fileBuffer.data(), fileBuffer.size());
				}
			}

			fclose(fp);
			finalize(MD5);

			String md5String;
			for (int i = 0; i < 16; ++i)
				md5String += toString(MD5.digest[i], IntegralBase::HEXADECIMAL);
			return md5String;
		}
	}
}