#include "Utilities\Timer.h"
#include "Utilities\String.h"
#include "Utilities\Exception.h"
#include "Utilities\ErrorMessage.h"
#include <Windows.h>
#include <string>

namespace kry
{
	namespace Util
	{
		void Timer::init()
		{
			LARGE_INTEGER large;

			if (!QueryPerformanceFrequency(&large))
				throw Util::Exception("Failed to initalise performance timer! " + Util::parseWinError(GetLastError()), KRY_EXCEPTION_DATA);

			frequency = double(large.QuadPart) / 1000.0;
		}

		Timer::Timer()
		{
			init();
			setDuration(0);
		}

		Timer::Timer(double Duration)
		{
			init();
			setDuration(Duration);
		}

		void Timer::setDuration(double Duration)
		{
			_delta = 0;
			duration = Duration;
			LARGE_INTEGER large;
			QueryPerformanceCounter(&large);
			start = large.QuadPart;
		}

		double Timer::getDuration()
		{
			return duration;
		}

		bool Timer::elapsed()
		{
			if (duration > 0)
				return (milliseconds() >= duration);

			return false;
		}

		double Timer::delta()
		{
			double ms = milliseconds();
			double time = ms - _delta;
			_delta = ms;

			return time;
		}

		double Timer::milliseconds()
		{
			LARGE_INTEGER large;
			QueryPerformanceCounter(&large);

			return double(large.QuadPart - start) / frequency;
		}

		double Timer::seconds()
		{
			return double(milliseconds() / 1000.0);
		}

		double Timer::minutes()
		{
			return double(seconds() / 60.0);
		}

		double Timer::hours()
		{
			return double(minutes() / 60.0);
		}
	}
}