#include "Audio\DeviceContext.h"
#include "Audio\Buffer.h"
#include "Audio\Listener.h"
#include "Audio\Source.h"
#include "Utilities\String.h"
#include <iostream>
#include <fstream>
#include <string>
#include <Windows.h>

using namespace std;
using namespace kry;
using namespace Audio;
using namespace Util;

void pause()
{
	std::cout << "\nPress any key to continue . . .";
	std::cin.get();
}

int endWithError(char* msg, int error=0)
{
    cout << msg << "\n";
	pause();
    return error;
}

std::vector<unsigned char> loadWav(const char* filename, Format &format, unsigned &frequency)
{
    FILE *fp = 0;
    fopen_s(&fp, filename,"rb");
    if (!fp) throw endWithError((char*)(string("Failed to open file ") + filename).c_str());

    char type[4];
    unsigned long size,chunkSize;
    short formatType,channels;
    unsigned long sampleRate,avgBytesPerSec;
    short bytesPerSample,bitsPerSample;
    unsigned long dataSize;
	
    fread(type,sizeof(char),4,fp);
    if(type[0]!='R' || type[1]!='I' || type[2]!='F' || type[3]!='F')
    throw endWithError ("No RIFF");

    fread(&size, sizeof(DWORD),1,fp);
    fread(type, sizeof(char),4,fp);
    if (type[0]!='W' || type[1]!='A' || type[2]!='V' || type[3]!='E')
    throw endWithError("not WAVE");

    fread(type,sizeof(char),4,fp);
    if (type[0]!='f' || type[1]!='m' || type[2]!='t' || type[3]!=' ')
    throw endWithError("not fmt ");

    fread(&chunkSize,sizeof(DWORD),1,fp);
    fread(&formatType,sizeof(short),1,fp);
    fread(&channels,sizeof(short),1,fp);
    fread(&sampleRate,sizeof(DWORD),1,fp);
    fread(&avgBytesPerSec,sizeof(DWORD),1,fp);
    fread(&bytesPerSample,sizeof(short),1,fp);
    fread(&bitsPerSample,sizeof(short),1,fp);

    fread(type,sizeof(char),4,fp);
    if (type[0]!='d' || type[1]!='a' || type[2]!='t' || type[3]!='a')
    throw endWithError("Missing DATA");

    fread(&dataSize,sizeof(DWORD),1,fp);
	/*
    cout << "Chunk Size: " << chunkSize << "\n";
    cout << "Format Type: " << formatType << "\n";
    cout << "Channels: " << channels << "\n";
    cout << "Sample Rate: " << sampleRate << "\n";
    cout << "Average Bytes Per Second: " << avgBytesPerSec << "\n";
    cout << "Bytes Per Sample: " << bytesPerSample << "\n";
    cout << "Bits Per Sample: " << bitsPerSample << "\n";
    cout << "Data Size: " << dataSize << "\n";
	*/
	std::vector<unsigned char> vec(dataSize);
	vec.resize(dataSize);
	cout << fread(vec.data(), sizeof(BYTE), dataSize, fp) << " bytes loaded\n";
    fclose(fp);  

	format = Format::UNKNOWN;
	if (bitsPerSample == 8)
	{
		if (channels == 1)
			format = Format::MONO_8;
		else if (channels == 2)
			format = Format::STEREO_8;
	}
    else if(bitsPerSample == 16)
    {
        if(channels == 1)
            format = Format::MONO_16;
        else if(channels == 2)
            format = Format::STEREO_16;
    }
	frequency = sampleRate;

	return vec;
}

int main(int argc, char* argv[])
{
	if (argc != 3)
		return endWithError("Usage: *.exe <musicfilename> <soundfilename>");
	Format musicformat, soundformat;
	unsigned musicfreq, soundfreq;
	auto& musicvec = loadWav(argv[1], musicformat, musicfreq);
	auto& soundvec = loadWav(argv[2], soundformat, soundfreq);

	DeviceContext::createDeviceContext();
	Listener::createListener();

	Buffer musicbuf(musicvec, musicformat, musicfreq);
	Buffer soundbuf(soundvec, soundformat, soundfreq);
	Source music1(musicbuf);
	Source music2(musicbuf);
	Source sound(soundbuf);

	music1.play();
	Sleep(200);
	music2.play();

	cout << "Press enter to play sound. Press space + enter to stop.\n";
	string in = " ";
	do
	{
		getline(cin, in);
		if (in.length() == 0)
			sound.play();
	} while (in.length() == 0);
	pause(); // wait for music to stop
}
