#include <iostream>
#include "System\Window.h"
#include "Graphics\Graphics.h"
#include "Utilities\BasicString.h"
#include <Windows.h>
/*#include <GL\glew.h>
#include <GL\gl.h>
#include <GL\glu.h>*/

void pause()
{
	std::cout << "\nEnd...\n";
}

int main()
{
	using namespace kry;
	using namespace Util;
	using namespace System;
	using namespace Graphics;

	try
	{
		bool running = true;
		Window window(stringLiteral("Test"), Vector2i(800, 600));
		WindowEvent event;

		Vector2i dimensions = window.getSize();

		// graphics stuff
		Renderer myRenderer;
		Canvas myCanvas, background;
		Quad myQuad, backdrop;
		Line myLine;
		Graphics::Ellipse myCircle; // Windows.h has a function called Ellipse
		Graphics::Ellipse myOval;
		Triangle myTriBorder, myTriFill;
		Font myFont;
		Text myText;

		myRenderer.addCanvas(&background);
		myRenderer.addCanvas(&myCanvas);

		myCanvas.dimensions = dimensions;
		myCanvas.setResolution(dimensions);
		myCanvas.addQuad(&myQuad);
		myCanvas.addLine(&myLine);
		myCanvas.addEllipse(&myCircle);
		myCanvas.addEllipse(&myOval);
		myCanvas.addTriangle(&myTriFill);
		myCanvas.addTriangle(&myTriBorder);

		myQuad.dimensions = Vector2f(100, 100);
		myQuad.position = (dimensions - myQuad.dimensions) / 2;
		myQuad.rgba = Vector4f(1, 1, 0, 1);

		myLine.points[0] = Vector2f(10, 10);
		myLine.points[1] = Vector2f(500, 500);

		myCircle.radius = Vector2f(60, 60);
		myCircle.position = (dimensions - myCircle.radius) / 2;
		myCircle.rgba = Vector4f(0.2, 0.3, 1, 1);
		myCircle.thickness = 1;

		myOval.position = Vector2f(500, 300);
		myOval.radius = Vector2f(50, 130);
		myOval.rgba = Vector4f(1, 0.3, 0, 1);

		myTriBorder.points[0] = Vector2f(100, 400);
		myTriBorder.points[1] = Vector2f(300, 100);
		myTriBorder.points[2] = Vector2f(150, 100);
		myTriBorder.thickness = 4;
		myTriBorder.rgba = Vector4f(1, 0, 0.2, 0.7);

		myTriFill.points[0] = myTriBorder.points[0];
		myTriFill.points[1] = myTriBorder.points[1];
		myTriFill.points[2] = myTriBorder.points[2];
		myTriFill.rgba = Vector4f(0, 1, 0.2, 1);

		myFont.loadFontFile("lucon.ttf", 70);

		myText.face = &myFont;
		myText.text = "a b c d e f g h i j k l m n o p q r s t u v w x y z A B C D E F G H I J K L M N O P Q R S T U V W X Y Z 0 1 2 3 4 5 6 7 8 9 ~ ! @ # $ % ^ & * ( ) _ + [ { ] } ; : ' \" , < . > / ?";
		myText.position = Vector2f(0, 60);
		//myText.scale = Vector2f(5.7);
		myCanvas.addText(&myText);		

		background.dimensions = dimensions;
		background.setResolution(dimensions);
		background.addQuad(&backdrop);
		backdrop.dimensions = dimensions;
		backdrop.rgba = Vector4f(0, 0, 0, 1);

		bool scaleCanvas = false;
		Vector2f pan;
		float panSpeed = 4;
		bool keyMem[4] = { false, false, false, false };

		while (running)
		{
			while (window.getEvent(event))
			{
				if (event.type == EventType::CLOSE)
					running = event.open;
				else if (event.type == EventType::KEY)
				{
					if (event.key == Key::ESCAPE)
						running = false;

					if (event.key == Key::W) keyMem[0] = event.down;
					if (event.key == Key::S) keyMem[1] = event.down;
					if (event.key == Key::A) keyMem[2] = event.down;
					if (event.key == Key::D) keyMem[3] = event.down;
				}
				else if (event.type == EventType::SIZE)
				{
					myCanvas.dimensions = window.getSize();
					background.dimensions = window.getSize();
					myCanvas.setResolution(window.getSize());
					background.setResolution(window.getSize());
				}
			}

			// camera movement
			if (keyMem[0]) pan[1] += panSpeed;
			if (keyMem[1]) pan[1] -= panSpeed;
			if (keyMem[2]) pan[0] += panSpeed;
			if (keyMem[3]) pan[0] -= panSpeed;

			// graphics stuff
			dimensions = window.getSize();

			myQuad.position[0] += 1;
			myQuad.rotation += 1;
			myOval.rotation += 1;
			myTriFill.rotation += 1;
			myTriBorder.rotation = myTriFill.rotation;
			myText.position[0] -= 0.5;

			// changing canvas rotation
			//myCanvas.setRotation(myCanvas.getRotation() + 1);
			Vector2f canDim = myCanvas.dimensions;

			if (scaleCanvas)
				canDim -= Vector2f(5, 5);
			else
				canDim += Vector2f(5, 5);

			if (canDim[0] < 100 || canDim[1] < 100)
				scaleCanvas = !scaleCanvas;
			else if (canDim[0] > dimensions[0] * 1.8 || canDim[1] > dimensions[1] * 1.8)
				scaleCanvas = !scaleCanvas;

			// changing canvas dimensions
			//myCanvas.setDimensions(canDim);

			if (myQuad.position[0] > canDim[0] - myQuad.dimensions[0])
				myQuad.position[0] = 0;

			if (myText.position[0] + myText.dimensions[0] < 0)
				myText.position[0] = canDim[0];

			myCanvas.pan = pan;

			myRenderer.render(window.getContext(), window.getGLContext(), window.getSize());
		}
		window.destroy();
	}
	catch (const Exception& e)
	{
		std::cout << e.what() << std::endl;
		std::cin.get();
	}

	pause();
}
