#include <iostream>
#include "System\Window.h"
#include "Graphics\Graphics.h"
#include "Utilities\BasicString.h"
#include "Media\ImageFactory.h"
#include <Windows.h>

void pause()
{
	std::cout << "\nEnd...\n";
	std::cin.get();
}

int main()
{
	using namespace kry;
	using namespace Util;
	using namespace System;
	using namespace Graphics;

	try
	{
		bool running = true;
		Window window(stringLiteral("Test"), Vector2i(800, 600));
		WindowEvent event;

		Vector2i dimensions = window.getSize();

		// graphics stuff
		Renderer myRenderer;
		Canvas myCanvas, background;
		Quad backdrop;
		Sprite mySprite, mySprite2;
		Font myFont;
		Text myText;

		myRenderer.addCanvas(&background);
		myRenderer.addCanvas(&myCanvas);

		myCanvas.dimensions = dimensions;
		myCanvas.setResolution(dimensions);
		myCanvas.addSprite(&mySprite);
		myCanvas.addSprite(&mySprite2);
		myCanvas.scale = Vector2f(2.5);

		background.dimensions = dimensions;
		background.setResolution(dimensions);
		background.addQuad(&backdrop);
		background.addSprite(&mySprite);
		backdrop.dimensions = dimensions;
		backdrop.rgba = Vector4f(0, 0.1, 0.2, 1);
		//backdrop.rgba = Vector4f(1);

		myFont.loadFontFile("lucon.ttf",  15);

		myText.face = &myFont;
		myText.text = "a b c d e f g h i j k l m n o p q r s t u v w x y z A B C D E F G H I J K L M N O P Q R S T U V W X Y Z 0 1 2 3 4 5 6 7 8 9 ~ ! @ # $ % ^ & * ( ) _ + [ { ] } ; : ' \" , < . > / ?";
		myText.position = Vector2f(0, 60);
		//myText.scale = Vector2f(5.7);
		myCanvas.addText(&myText);

		Vector2f pan;
		float panSpeed = 4;
		bool keyMem[4] = { false, false, false, false };

		// PNG filefactory stuff
		Texture pngTex = Media::imageFileToTexture("image.png");

		// JPG
		Vector2i jpgdims;
		auto jpgrgba = Media::imageFileToRGBA("image.jpg", jpgdims);
		Texture jpgTex;
		jpgTex.loadRGBA(jpgrgba, jpgdims);
		//Texture jpgTex = Media::imageFileToTexture("image.jpg");

		mySprite.texture = &pngTex;
		mySprite.dimensions = pngTex.getDimensions() *  Vector2f(3);
		mySprite.position = Vector2f(-100, 0);
		mySprite2.texture = &jpgTex;
		mySprite2.dimensions = jpgTex.getDimensions();
		std::cout << mySprite.dimensions << " " << mySprite.texture->getID() << "\n";

		while (running)
		{
			while(window.getEvent(event))
			{
				if (event.type == EventType::CLOSE)
					running = event.open;
				else if (event.type == EventType::KEY)
				{
					if (event.key == Key::ESCAPE)
						running = false;				

					if (event.key == Key::W) keyMem[0] = event.down;
					if (event.key == Key::S) keyMem[1] = event.down;
					if (event.key == Key::A) keyMem[2] = event.down;
					if (event.key == Key::D) keyMem[3] = event.down;
				}
				else if (event.type == EventType::SIZE)
				{
					std::cout << "Window resized" << std::endl;
					myCanvas.dimensions = window.getSize();
					//background.dimensions = Vector2f(window.getSize()) * Vector2f(0.5);
					myCanvas.setResolution(window.getSize());
					background.setResolution(Vector2f(window.getSize()) * Vector2f(0.5));
				}
			}

			// camera movement
			if (keyMem[0]) pan[1] += panSpeed;
			if (keyMem[1]) pan[1] -= panSpeed;
			if (keyMem[2]) pan[0] += panSpeed;
			if (keyMem[3]) pan[0] -= panSpeed;

			// graphics stuff
			myCanvas.pan = pan;

			myRenderer.render(window.getContext(), window.getGLContext(), window.getSize());
			myRenderer.clearBuffer(Vector4f(0, 0, 0, 1));
		}
		window.destroy();
	}
	catch (const Exception& e)
	{
		std::cout << e.what() << std::endl;
	}

	//pause();
}
