#include <iostream>
#include "Utilities\String.h"
#include <string>

/*
To compile, you'll need:
Utilities\BasicString.h
Utilities\StringTypes.h
Utilities\String.h
*/

/*
Mark these off as you go.
+ = passed all tests
- = failed at least 1 test (mention which one in a comment)

[ ] String(); // default
[ ] String(const char[] string); // try your best not to use this, its for noob coders, there are better alternatives (maybe deprecate)
[ ] String(const char* string, unsigned length); // copy buffer
[ ] String(unsigned length, char fill); // fill
[ ] String(unsigned capacity); // initialise capacity
[ ] String(const String& copy); // copy
[ ] String(String&& move); // move
[ ] long indexOf(const String& tofind, unsigned index = 0) const; // cast the result to an unsigned to get the actual index, if it's not -1
[ ] long indexOf(char tofind, unsigned index = 0) const;
[ ] long lastIndexOf(const String& tofind) const;
[ ] long lastIndexOf(const String& tofind, unsigned index) const;
[ ] long lastIndexOf(char tofind) const;
[ ] long lastIndexOf(char tofind, unsigned index) const;
[ ] String substring(unsigned index) const;
[ ] String substring(unsigned index, unsigned length) const;
[ ] String replace(const String& toreplace, const String& replacewith) const;
[ ] String replace(char toreplace, char replacewith) const;
[ ] bool contains(const String& totest, unsigned index = 0) const;
[ ] bool contains(char totest, unsigned index = 0) const;
[ ] bool startsWith(const String& totest) const;
[ ] bool endsWith(const String& totest) const;
[ ] std::vector<String> explode(const String& delim) const;
[ ] std::vector<String> explode(char delim) const;
[ ] String insert(const String& toinsert, unsigned index) const;
[ ] String insert(char toinsert, unsigned index) const;
[ ] String reverse() const;
[ ] String resize(unsigned newlength) const;
[ ] void shrink();
[ ] void clear();
[ ] String trim() const;
[ ] String& operator=(const String& copy);
[ ] String& operator=(String&& move);
[ ] String& operator+=(const String& right);
[ ] String& operator+=(char right);
[ ] char& operator[](unsigned index) const;
[ ] const char* getData() const;
[ ] unsigned getLength() const;
[ ] unsigned getCapacity() const;
[ ] bool isEmpty() const;
*/
using namespace kry;
using namespace Util;

void pause()
{
	std::cout << "\nPress any key to continue...\n";
	std::cin.get();
}

void p_assert(bool assertion, const char* functionName)
{
	std::cout << (assertion ? "[+] " : "[-] ") << functionName << '\n';
}

int main()
{
	try
	{
		{
			String test(" ");
			p_assert(test.trim() == "", "trim one space");
		}
		{
			String test("    ");
			p_assert(test.trim() == "", "trim multiple spaces");
		}
		{
			String test("");
			p_assert(test.trim() == "", "trim no spaces");
		}
		{
			String test(" hello ");
			p_assert(test.trim() == "hello", "trim edge spaces 1");
		}
		{
			String test("   hello   ");
			p_assert(test.trim() == "hello", "trim edge spaces 3");
		}
		{
			String test(" hello   ");
			p_assert(test.trim() == "hello", "trim edge spaces 1-3");
		}
		{
			String test("   hello ");
			p_assert(test.trim() == "hello", "trim edge spaces 3-1");
		}
		{
			String test("hello   ");
			p_assert(test.trim() == "hello", "trim edge spaces 0-3");
		}
		{
			String test("   hello");
			p_assert(test.trim() == "hello", "trim edge spaces 3-0");
		}
		{
			String test("hello");
			p_assert(test.trim() == "hello", "trim no spaces with length");
		}
		{
			String test("h");
			p_assert(test.trim() == "h", "trim no spaces with length 1");
		}
		{
			String test(" h");
			p_assert(test.trim() == "h", "trim left space with length 2");
		}
		{
			String test("h ");
			p_assert(test.trim() == "h", "trim right space with length 2");
		}
		{
			String test("   h");
			p_assert(test.trim() == "h", "trim left space with length 4");
		}
		{
			String test("h   ");
			p_assert(test.trim() == "h", "trim right space with length 4");
		}
	}
	catch (const Exception& e)
	{
		std::cout << e.what() << std::endl;
	}

	/*
	{
		String test1("this is a test");
		String test2("this is a test");
		p_assert(test1 == test2, "== same length");
		std::string stest1("this is a test");
		std::string stest2("this is a test");
		p_assert(stest1 == stest2, "== same length std");
	}
	{
		String test1("this is a test1");
		String test2("this is a test");
		p_assert(test1 == test2, "== 1 greater");
		std::string stest1("this is a test1");
		std::string stest2("this is a test");
		p_assert(stest1 == stest2, "== 1 greater std");
	}
	{
		String test1("this is a test");
		String test2("this is a test1");
		p_assert(test1 == test2, "== 2 greater");
		std::string stest1("this is a test");
		std::string stest2("this is a test1");
		p_assert(stest1 == stest2, "== 2 greater std");
	}
	{
		String test1("this is a test");
		String test2("this is a test");
		p_assert(test1 >= test2, ">= same length");
		std::string stest1("this is a test");
		std::string stest2("this is a test");
		p_assert(stest1 >= stest2, ">= same length std");
	}
	{
		String test1("this is a test1");
		String test2("this is a test");
		p_assert(test1 >= test2, ">= 1 greater");
		std::string stest1("this is a test1");
		std::string stest2("this is a test");
		p_assert(stest1 >= stest2, ">= 1 greater std");
	}
	{
		String test1("this is a test");
		String test2("this is a test1");
		p_assert(test1 >= test2, ">= 2 greater");
		std::string stest1("this is a test");
		std::string stest2("this is a test1");
		p_assert(stest1 >= stest2, ">= 2 greater std");
	}
	{
		String test1("this is a test");
		String test2("this is a test");
		p_assert(test1 <= test2, "<= same length");
		std::string stest1("this is a test");
		std::string stest2("this is a test");
		p_assert(stest1 <= stest2, "<= same length std");
	}
	{
		String test1("this is a test1");
		String test2("this is a test");
		p_assert(test1 <= test2, "<= 1 greater");
		std::string stest1("this is a test1");
		std::string stest2("this is a test");
		p_assert(stest1 <= stest2, "<= 1 greater std");
	}
	{
		String test1("this is a test");
		String test2("this is a test1");
		p_assert(test1 <= test2, "<= 2 greater");
		std::string stest1("this is a test");
		std::string stest2("this is a test1");
		p_assert(stest1 <= stest2, "<= 2 greater std");
	}
	{
		String test1("this is a test");
		String test2("this is a test");
		p_assert(test1 > test2, "> same length");
		std::string stest1("this is a test");
		std::string stest2("this is a test");
		p_assert(stest1 > stest2, "> same length std");
	}
	{
		String test1("this is a test1");
		String test2("this is a test");
		p_assert(test1 > test2, "> 1 greater");
		std::string stest1("this is a test1");
		std::string stest2("this is a test");
		p_assert(stest1 > stest2, "> 1 greater std");
	}
	{
		String test1("this is a test");
		String test2("this is a test1");
		p_assert(test1 > test2, "> 2 greater");
		std::string stest1("this is a test");
		std::string stest2("this is a test1");
		p_assert(stest1 > stest2, "> 2 greater std");
	}
	{
		String test1("this is a test");
		String test2("this is a test");
		p_assert(test1 < test2, "< same length");
		std::string stest1("this is a test");
		std::string stest2("this is a test");
		p_assert(stest1 < stest2, "< same length std");
	}
	{
		String test1("this is a test1");
		String test2("this is a test");
		p_assert(test1 < test2, "< 1 greater");
		std::string stest1("this is a test1");
		std::string stest2("this is a test");
		p_assert(stest1 < stest2, "< 1 greater");
	}
	{
		String test1("this is a test");
		String test2("this is a test1");
		p_assert(test1 < test2, "< 2 greater");
		std::string stest1("this is a test");
		std::string stest2("this is a test1");
		p_assert(stest1 < stest2, "< 2 greater");
	}
	{
		p_assert(toString(100, IntegralBase::HEXADECIMAL) == "a", "toString hex");
		std::cout << toString(100, IntegralBase::BINARY) << std::endl;
		std::cout << toString(100, IntegralBase::DECIMAL) << std::endl;
		std::cout << toString(100, IntegralBase::OCTAL) << std::endl;
		std::cout << toString(100, IntegralBase::HEXADECIMAL) << std::endl;
		std::cout << toString(123) << std::endl;
	}
	*/
	pause();
}